@@ -16,7 +16,10 @@ import numpy as np
 from scipy.optimize import minimize
 from scipy.ndimage import fourier_shift
 from scipy.fftpack import fftn, ifftn, fftshift
-from skimage.feature import register_translation
+try:
+    from skimage.registration import phase_cross_correlation as register_translation
+except ImportError:
+    from skimage.feature import register_translation
 from ..processing_unit import has_cuda, has_opencl
 from ..processing_unit import default_processing_unit as main_default_processing_unit
 import warnings
@@ -26,10 +29,14 @@ import warnings
 #     import pycuda.gpuarray as cua
 #     from pycuda.elementwise import ElementwiseKernel as CU_ElK
 #     from pycuda.reduction import ReductionKernel as CU_RedK
-if has_opencl and main_default_processing_unit.pu_language not in ['cuda', 'cpu']:
-    import pyopencl as cl
+
+if has_cuda and main_default_processing_unit.pu_language not in ['opencl', 'cpu']:
+    import pycuda.gpuarray as cua
+    from pycuda.reduction import ReductionKernel as CU_RedK
+    from ..processing_unit.cu_processing_unit import CUProcessingUnit
+    from ..processing_unit.kernel_source import get_kernel_source as getks
+elif has_opencl and main_default_processing_unit.pu_language not in ['cuda', 'cpu']:
     import pyopencl.array as cla
-    from pyopencl.elementwise import ElementwiseKernel as CL_ElK
     from pyopencl.reduction import ReductionKernel as CL_RedK
     from pyopencl import CompilerWarning
 
@@ -169,6 +176,24 @@ def fit_phase2_cl(pars, a1, a2, cl_func, cl_queue):
     return r
 
 
+def fit_phase2_cu(pars, a1, a2, cu_func):
+    """
+    Fit function to match the phase between two arrays, with linear phase shift parameters. OpenCL version
+    :param pars: a list of ndim+1 linear phase shift parameters (one constant plus one along each axis)
+    :param a1: the amplitude (or weight) of the first array. Should be >=0
+    :param a2:  the amplitude (or weight) of the second array. Should be >=0
+    :return: a floating point figure of merit, ((a1 + a2) * delta_phi ** 2).sum()
+    """
+    nx = np.int32(a1.shape[-1])
+    ny = np.int32(a1.shape[-2])
+    if a1.ndim == 3:
+        nz = np.int32(a1.shape[0])
+        r = cu_func(a1, a2, cua.to_gpu(np.array(pars).astype(np.float32)), nx, ny, nz).get()
+    else:
+        r = cu_func(a1, a2, cua.to_gpu(np.array(pars).astype(np.float32)), nx, ny).get()
+    return r
+
+
 def r_match(d1: np.ndarray, d2: np.ndarray, percent=99, threshold=0.05):
     """
     Compute an unweighted R-factor between two arrays (complex or real)
@@ -289,7 +314,49 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
             print("Match2: dt[register_translation]=%6.3fs" % (t3 - t2))
 
         # Match phase shift and ramp, testing both conjugates
-        if has_opencl and use_gpu:
+        if has_cuda and use_gpu:
+            if verbose:
+                print("Match2: match_phase_ramp using CUDA")
+            if main_default_processing_unit.cu_device is None:
+                main_default_processing_unit.select_gpu(language='cuda')
+            if match2.pu is None:
+                match2.pu = CUProcessingUnit()
+                match2.pu.init_cuda(cu_device=main_default_processing_unit.cu_device, test_fft=False, verbose=False)
+            if d1.ndim == 2:
+                if match2.cu_dphi_red2 is None:
+                    match2.cu_dphi_red2 = CU_RedK(np.float32, neutral="0", reduce_expr="a+b",
+                                                  map_expr="DeltaPhi2(i, a1, a2, c, nx, ny)",
+                                                  preamble=getks('cuda/complex.cu') +
+                                                           getks("cdi/cuda/delta_phi_red.cu"),
+                                                  options=["-use_fast_math"],
+                                                  arguments="pycuda::complex<float> *a1, pycuda::complex<float>  *a2,"
+                                                            "float *c,  const int nx, const int ny")
+                cu_dphi_red = match2.cu_dphi_red2
+            else:
+                if match2.cu_dphi_red3 is None:
+                    match2.cu_dphi_red3 = CU_RedK(np.float32, neutral="0", reduce_expr="a+b",
+                                                  map_expr="DeltaPhi3(i, a1, a2, c, nx, ny, nz)",
+                                                  preamble=getks('cuda/complex.cu') +
+                                                           getks("cdi/cuda/delta_phi_red.cu"),
+                                                  options=["-use_fast_math"],
+                                                  arguments="pycuda::complex<float> *a1, pycuda::complex<float>  *a2,"
+                                                            "float *c,  const int nx, const int ny, const int nz")
+                cu_dphi_red = match2.cu_dphi_red3
+
+            cu_d1 = cua.to_gpu(d1.astype(np.complex64))
+            cu_d2 = cua.to_gpu(d2.astype(np.complex64))
+
+            p0 = np.zeros(1 + d1.ndim)
+            p0[0] = dphi
+            p0 = minimize(fit_phase2_cu, p0, args=(cu_d1, cu_d2, cu_dphi_red), method='powell')
+            if verbose >= 3:
+                print(p0)
+            cu_d2 = cua.to_gpu(d2.conj().astype(np.complex64))
+            p1 = minimize(fit_phase2_cu, np.zeros(1 + d1.ndim), args=(cu_d1, cu_d2, cu_dphi_red), method='powell')
+            if verbose >= 3:
+                print(p1)
+            del cu_d1, cu_d2
+        elif has_opencl and use_gpu:
             if verbose:
                 print("Match2: match_phase_ramp using OpenCL")
             if main_default_processing_unit.cl_device is None:
@@ -397,3 +464,5 @@ def match2(d1: np.ndarray, d2: np.ndarray, match_phase_ramp=False, match_orienta
 match2.pu = None
 match2.cl_dphi_red2 = None
 match2.cl_dphi_red3 = None
+match2.cu_dphi_red2 = None
+match2.cu_dphi_red3 = None
