@@ -110,7 +110,7 @@ def make_cdi_data_file(shape=(128, 128, 128), obj_shape='rectangle', file_type='
     mask[np.random.uniform(0, 1, mask.shape) < mask_fraction] = True
 
     if file_type == 'cxi':
-        f, path = tempfile.mkstemp(suffix='.cxi', dir=dir)
+        f, path = tempfile.mkstemp(suffix='.cxi', prefix="TestCDI", dir=dir)
         save_cdi_data_cxi(path, d, wavelength=1.5e-10, detector_distance=1, pixel_size_detector=55e-6, mask=mask,
                           sample_name=None, experiment_id=None, instrument=None, note=None, iobs_is_fft_shifted=False)
     else:
@@ -121,6 +121,25 @@ def make_cdi_data_file(shape=(128, 128, 128), obj_shape='rectangle', file_type='
     return path
 
 
+def make_cdi_mask_file(shape=(128, 128, 128), file_type='npz', dir=None, fraction_mask=0.01):
+    """
+    Create a mask file.
+    :param shape: the shape of the data file, either 2D or 3D.
+    :param file_type: can be npz, npy.
+    :param dir: the directory where the file will be created
+    :param fraction_mask: fraction of masked pixels, which will be randomly distributed
+    :return: the file name
+    """
+    mask = np.random.uniform(0, 1, shape) < fraction_mask
+    # Use mixed case for the name to test no lowercase is enforced
+    f, path = tempfile.mkstemp(suffix='.' + file_type, prefix="TestMask", dir=dir)
+    if file_type == "npy":
+        np.save(path, mask, allow_pickle=False)
+    else:
+        np.savez_compressed(path, mask=mask)
+    return path
+
+
 class TestCDI(unittest.TestCase):
 
     @classmethod
@@ -265,6 +284,45 @@ class TestCDI(unittest.TestCase):
             # print("HIO**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
             cdi = op.FreePU() * cdi
 
+    @unittest.skipIf('cuda' in sys.argv or exclude_opencl, "OpenCL tests excluded")
+    @unittest.skipIf(has_opencl is False, 'no OpenCL support')
+    def test_ER_PSF_opencl(self):
+        self.make_cdi_obj2d()
+        self.make_cdi_obj3d()
+        op = clop
+        for cdi, obj0, support0 in [(self.cdi_2d, self.obj0_2d, self.support0_2d),
+                                    (self.cdi_3d, self.obj0_3d, self.support0_3d)]:
+            cdi.set_obj(obj0)
+            cdi.set_support(support0)
+            # Compute LLK before and after
+            cdi = op.LLK() * cdi
+            llk0 = cdi.get_llkn()
+            self.p = op.ER(positivity=True, calc_llk=0) ** 10 * cdi
+            self.p = op.LLK() * op.ER(positivity=True, calc_llk=0) ** 10 * op.EstimatePSF()**20 * cdi
+            llk1 = cdi.get_llkn()
+            # print("HIO**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
+            cdi = op.FreePU() * cdi
+
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_ER_PSF_cuda(self):
+        self.make_cdi_obj2d()
+        self.make_cdi_obj3d()
+        op = cuop
+        for cdi, obj0, support0 in [(self.cdi_2d, self.obj0_2d, self.support0_2d),
+                                    (self.cdi_3d, self.obj0_3d, self.support0_3d)]:
+            cdi.set_obj(obj0)
+            cdi.set_support(support0)
+            # Compute LLK before and after
+            cdi = op.LLK() * cdi
+            llk0 = cdi.get_llkn()
+            self.p = op.ER(positivity=True, calc_llk=0) ** 10 * cdi
+            self.p = op.LLK() * op.ER(positivity=True, calc_llk=0) ** 10 * op.EstimatePSF()**20 * cdi
+            llk1 = cdi.get_llkn()
+            # print("HIO**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
+            cdi = op.FreePU() * cdi
+
+
     @unittest.skipIf('cuda' in sys.argv or exclude_opencl, "OpenCL tests excluded")
     @unittest.skipIf(has_opencl is False, 'no OpenCL support')
     def test_RAAR_opencl(self):
@@ -346,6 +404,8 @@ class TestCDI(unittest.TestCase):
             # print("HIO**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
             cdi = op.FreePU() * cdi
 
+    @unittest.skipIf('cuda' in sys.argv or exclude_opencl, "OpenCL tests excluded")
+    @unittest.skipIf(has_opencl is False, 'no OpenCL support')
     def test_InterpIobsMask_opencl(self):
         self.make_cdi_obj2d()
         self.make_cdi_obj3d()
@@ -365,6 +425,8 @@ class TestCDI(unittest.TestCase):
             llk1 = cdi.get_llkn()
             cdi = op.FreePU() * cdi
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
     def test_InterpIobsMask_cuda(self):
         self.make_cdi_obj2d()
         self.make_cdi_obj3d()
@@ -443,7 +505,7 @@ class TestCDI(unittest.TestCase):
         atol = np.abs(objcl).max() * atol
         nb = (supcl != supcu).sum()
         nbf = nb / obj0.size * 100
-        self.assertTrue((nb < nbfractol*obj0.size),
+        self.assertTrue((nb < nbfractol * obj0.size),
                         msg="%s and %s operation (%dD): '%s' must give close results (dSupport=%d [%4.2f%%])" %
                             (pu1, pu2, ndim, op, nb, nbf))
         nb = (abs(objcu - objcl) > (rtol * abs(objcl) + atol)).sum()
