@@ -286,6 +286,14 @@ class HoloTomo:
         self._probe = probe.copy().astype(np.complex64)
         self._timestamp_counter += 1
 
+    def get_probe(self):
+        """
+        Get the probe
+        :return: the complex array of the probe, with 3 dimensions (1 for the modes)
+        """
+        self._from_pu()
+        return self._probe
+
     def set_obj(self, obj):
         """
         Give a new value for the object projections. The supplied array shape should be: (nb_proj, ny, nx)
@@ -412,18 +420,27 @@ class HoloTomo:
             if 'algorithm' in kwargs:
                 self.history.insert(self.cycle, algorithm=kwargs['algorithm'])
 
-    def save_obj_probe_chunk(self, chunk_prefix, save_obj_phase=True, save_obj_complex=False, dtype=np.float16):
+    def save_obj_probe_chunk(self, chunk_prefix, save_obj_phase=True, save_obj_complex=False,
+                             save_probe=True, dtype=np.float16, verbose=False):
         """
-        Save the chunk corresponding to this object in an hdf5 file
+        Save the chunk (the projections included in this object) in an hdf5 file
         :param chunk_prefix: the prefix, e.g. "my_sample_%04d" for the filename the data will be saved to.
             The %04d field will be replaced by the first projection. A '.h5' prefix will be appended
+            if no "%" is included in the prefix, only '.h5' will be appended to the filename.
         :param save_obj_phase: if True, save the object phase
         :param save_obj_complex: if True, also save the different complex projections of the object
+        :param save_probe: if True, save the complex probe (illumination)
         :param dtype: the floating point dtype to save the phase
+        :param verbose: if True, will print some information.
         :return:
         """
-        f = h5py.File(chunk_prefix % self.data.idx[0] + ".h5", "w")
-        f.create_dataset("cxi_version", data=150)
+        fname = chunk_prefix + ".h5"
+        if '%' in fname:
+            fname = fname % self.data.idx[0]
+        if verbose:
+            print("Saving holotomo projections chunk & illumination to: %s" % fname)
+        f = h5py.File(fname, "w")
+        # f.create_dataset("cxi_version", data=150)
         entry = f.create_group("/entry_1")
         entry_path = "/entry_1"
         f.attrs['default'] = 'entry_1'
@@ -539,6 +556,32 @@ class HoloTomo:
             result_2['col_coords'].attrs['units'] = unit_name
             result_2['col_coords'].attrs['long_name'] = 'X (%s)' % unit_name
 
+        if save_probe:
+            result_3 = entry.create_group("result_3")
+            entry["probe"] = h5py.SoftLink(entry_path + '/result_3')
+            result_3['title'] = 'Probe'
+            result_3.attrs['NX_class'] = 'NXdata'
+            result_3.attrs['signal'] = 'data'
+
+            result_3.create_dataset("data", data=self.get_probe(), chunks=True, shuffle=True, compression="gzip")
+            result_3["data"].attrs['interpretation'] = 'image'
+            result_3.create_dataset("data_space", data="real")
+            result_3.create_dataset("image_size", data=[px * nx, px * ny])
+            # Store probe pixel size (not in CXI specification)
+            result_3.create_dataset("x_pixel_size", data=px)
+            result_3.create_dataset("y_pixel_size", data=px)
+            # X & Y axis data for NeXuS plotting
+            result_3.attrs['axes'] = np.array(['row_coords', 'col_coords'], dtype=h5py.special_dtype(vlen=str))
+            # Flip to have origin at top, left
+            yc = np.flip((np.arange(ny) * px - ny * px / 2) * unit_scale)
+            result_3.create_dataset('row_coords', data=yc)
+            result_3['row_coords'].attrs['units'] = unit_name
+            result_3['row_coords'].attrs['long_name'] = 'Y (%s)' % unit_name
+            xc = (np.arange(nx) * px - nx * px / 2) * unit_scale
+            result_3.create_dataset('col_coords', data=xc)
+            result_3['col_coords'].attrs['units'] = unit_name
+            result_3['col_coords'].attrs['long_name'] = 'X (%s)' % unit_name
+
 
 class OperatorHoloTomo(Operator):
     """
