@@ -11,8 +11,10 @@ from .cuda_device import has_cuda
 from .opencl_device import has_opencl
 import numpy as np
 from scipy.fftpack import fftn, ifftn, fftshift
+
 try:
     from mpi4py import MPI
+
     mpi = MPI.COMM_WORLD
     if mpi.Get_size() == 1:
         mpi = None
@@ -242,9 +244,17 @@ class ProcessingUnit(object):
                     s = pynx_py_env[1]
                     try:
                         gpu_rank = int(s)
-                        ranking = 'order'
-                        if verbose:
-                            print("PyNX processing unit: use #%d device from PYNX_PU environment variable" % gpu_rank)
+                        if gpu_rank < 100:
+                            ranking = 'order'
+                            if verbose:
+                                print("PyNX processing unit: use #%d device from "
+                                      "PYNX_PU environment variable" % gpu_rank)
+                        else:
+                            # that's for the case where the card number (e.g. 1080) is given...
+                            gpu_name = s
+                            if verbose:
+                                print("PyNX processing unit: searching '%s' GPU from PYNX_PU environment variable" % s)
+
                     except ValueError:
                         gpu_name = s
                         if verbose:
