@@ -30,6 +30,7 @@ except ImportError:
 
 from . import ProcessingUnit
 from .cl_resources import cl_resources
+from .opencl_device import cl_device_fft_speed
 
 from pynx.utils.math import test_smaller_primes
 
@@ -81,6 +82,8 @@ class CLProcessingUnit(ProcessingUnit):
         self.profiling = False  # profiling enabled ?
         # Keep track of events for profiling
         self.cl_event_profiling = {}
+        # gpyfft plans
+        self._gpyfft_plan_v = {}
 
     def init_cl(self, cl_ctx=None, cl_device=None, fft_size=(1, 1024, 1024), batch=True, gpu_name=None, test_fft=True,
                 verbose=True, profiling=None):
@@ -117,10 +120,6 @@ class CLProcessingUnit(ProcessingUnit):
         if self.cl_ctx.devices[0].name.find('Iris Pro') >= 0:
             self.cl_options += " -DIRISPROBUG=1 "
         self.cl_init_kernels()
-        # Reset gpyfft plan
-        self.gpyfft_plan = None  # gpyfft plan
-        self.gpyfft_shape = None  # gpyfft data shape
-        self.gpyfft_axes = None  # gpyfft axes
 
     def get_context(device):
         """
@@ -153,20 +152,39 @@ class CLProcessingUnit(ProcessingUnit):
 
     def cl_fft_set_plan(self, cl_data, axes=(-1, -2)):
         """
-        Creates FFT plan, or updates it if the shape of the data or the axes have changed. Nothing is done
-        if the shape
+        Creates FFT plan, or updates it if the shape of the data or the axes have changed.
+
+        .. deprecated:: 2020.2
+        Use :func:`cl_fft_get_plan` instead.
+
         :param cl_data: the pyopencl.array.Array data
         :param axes: the FFT axes. If None, this will be set to (-2, -1) for 2D and (-3, -2, -1) for 3D
         :return: nothing
         """
+        warnings.warn("CLProcessingUnit.cl_fft_set_plan() is deprecated, use cl_fft_get_plan() instead",
+                      DeprecationWarning)
+        self.gpyfft_plan = self.cl_fft_get_plan(cl_data, axes=axes)
+
+    def cl_fft_get_plan(self, cl_data, axes=(-1, -2), shuffle_axes=False):
+        """
+        Get a FFT plan according to the given parameters. if the plan already exists, it is re-used
+
+        :param cl_data: the pyopencl.array.Array data
+        :param axes: the FFT axes. If None, this will be set to (-2, -1) for 2D and (-3, -2, -1) for 3D
+        :param shuffle_axes: if True, will try permutations on the axes order to get the optimal
+                             speed.
+        :return: nothing
+        """
+        # TODO: it would be better to store a plan independent of the actual data,
+        #  and then only use enqueue_arrays(..) instead of enqueue(..).
         if axes is None:
-            # Use default values providing good spped with gpyfft
+            # Use default values providing good speed with gpyfft
             if cl_data.ndim == 2:
                 axes = (-1, -2)
             else:
                 axes = (-1, -2, -3)
         elif len(axes) < cl_data.ndim - 1:
-            # Kludge to work around "data layout not supported (only single non-transformed axis allowd)" in gpyfft
+            # Kludge to work around "data layout not supported (only single non-transformed axis allowed)" in gpyfft
             s = list(cl_data.shape)
             n = cl_data.ndim
             for i in range(1, cl_data.ndim):
@@ -181,15 +199,27 @@ class CLProcessingUnit(ProcessingUnit):
             if s != cl_data.shape:
                 # print("cl_fft_set_plan: collapsing data shape from ", cl_data.shape, " to ", s, '. Axes:', axes)
                 cl_data = cl_data.reshape(s)
+        k = (cl_data.shape, cl_data.dtype, axes)
 
-        if self.gpyfft_plan is not None:
-            if cl_data.shape == self.gpyfft_shape and axes == self.gpyfft_axes:
-                self.gpyfft_plan.data = cl_data
-                return
+        if k in self._gpyfft_plan_v:
+            self._gpyfft_plan_v[k].data = cl_data
+            return self._gpyfft_plan_v[k]
 
-        self.gpyfft_plan = gpyfft.FFT(self.cl_ctx, self.cl_queue, cl_data, None, axes=axes)
-        self.gpyfft_shape = cl_data.shape
-        self.gpyfft_axes = axes
+        if shuffle_axes:
+            # Change the axes if necessary, but keep the original ones as key
+            flops, dt, axes = cl_device_fft_speed(self.cl_device, cl_data.shape, axes, timing=True, shuffle_axes=True)
+
+        self._gpyfft_plan_v[k] = gpyfft.FFT(self.cl_ctx, self.cl_queue, cl_data, None, axes=axes)
+
+        return self._gpyfft_plan_v[k]
+
+    def cl_fft_free_plans(self):
+        """
+        Delete the gpyfft plans from memory. Actual deletion may only occur later (gc...)
+        :return: nothing
+        """
+        self.finish()
+        self._gpyfft_plan_v = {}
 
     def finish(self):
         for e in self.ev:
