@@ -6,6 +6,7 @@
 #         Vincent Favre-Nicolin, favre@esrf.fr
 
 import warnings
+
 warnings.filterwarnings("ignore", message="creating CUBLAS context to get version number")
 import numpy as np
 
@@ -106,27 +107,45 @@ class CUProcessingUnit(ProcessingUnit):
                       DeprecationWarning)
         self.cufft_plan = self.cu_fft_get_plan(cu_data, batch, stream)
 
-    def cu_fft_get_plan(self, cu_data, batch=True, stream=None):
+    def cu_fft_get_plan(self, cu_data, batch=True, stream=None, dtype_dest=None, dtype_src=None):
         """
         Get a FFT plan according to the given parameters. if the plan already exists, it is re-used
 
-        :param cu_data: an array from which the FFT shape will be extracted
+        :param cu_data: an array from which the FFT shape will be extracted. Alternatively
+            this can be the shape of the array, but dtype_src and dtype_dest must be given
         :param batch: if True, perform a 2D batch FFT on an n-dimensional array (n>2). Ignored if data is 2D.
                       The FFT is computed over the last two dimensions.
         :param stream: the cuda stream to be associated with this plan
+        :param dtype_dest: if None, the destination array dtype is taken from cu_data.
+            This can be used for ccomplex<->real transforms
+        :param dtype_src: if None, the source array dtype is taken from cu_data.
+            This can be used for ccomplex<->real transforms
         :return: the cufft plan
         """
-        if len(cu_data.shape) == 2:
+        if isinstance(cu_data, tuple) or isinstance(cu_data, list):
+            fft_shape = cu_data
+        else:
+            fft_shape = cu_data.shape
+
+        if len(fft_shape) == 2:
             batch = False
-        k = (cu_data.shape, cu_data.dtype, batch, stream)
+
+        if dtype_dest is None:
+            dtype_dest = cu_data.dtype
+
+        if dtype_src is None:
+            dtype_src = cu_data.dtype
+
+        k = (fft_shape, dtype_src, dtype_dest, batch, stream)
+
         if k not in self._cufft_plan_v:
             if batch:
                 # print("Setting cufft plan: ", cu_data.shape, batch)
-                self._cufft_plan_v[k] = cu_fft.Plan(cu_data.shape[-2:], cu_data.dtype, cu_data.dtype,
+                self._cufft_plan_v[k] = cu_fft.Plan(fft_shape[-2:], dtype_src, dtype_dest,
                                                     batch=np.product(cu_data.shape[:-2]), stream=stream)
             else:
                 # print("Setting cufft plan: ", cu_data.shape, batch)
-                self._cufft_plan_v[k] = cu_fft.Plan(cu_data.shape, cu_data.dtype, cu_data.dtype, batch=1, stream=stream)
+                self._cufft_plan_v[k] = cu_fft.Plan(fft_shape, dtype_src, dtype_dest, batch=1, stream=stream)
 
         return self._cufft_plan_v[k]
 
@@ -135,7 +154,13 @@ class CUProcessingUnit(ProcessingUnit):
         Delete the cufft plans from memory. Actual deletion may only occur later (gc...)
         :return: nothing
         """
-        self._cufft_plan_v = {}
+        if cu_fft.cufft.cufftGetVersion() > 10010:
+            txt = "WARNING: cuFFT plan destruction inhibited as a workaround for " \
+                  "an issue with CUDA>=11.0. See https://github.com/lebedov/scikit-cuda/issues/308"
+            warnings.filterwarnings('once', message=txt, append=True)
+            warnings.warn(txt, UserWarning)
+        else:
+            self._cufft_plan_v = {}
 
     def enable_profiling(self, profiling=True):
         """
@@ -151,3 +176,7 @@ class CUProcessingUnit(ProcessingUnit):
 
     def finish(self):
         self.cu_ctx.synchronize()
+
+
+# This is mostly used during tests while avidoing to destroy & create FFT plans
+default_processing_unit = CUProcessingUnit()
