@@ -217,6 +217,43 @@ class TestCUDA(unittest.TestCase):
 
         self.assertTrue(fft_ok, msg=msg)
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    def test_pycuda_fft_3D_delete(self):
+        # Test the workaround to https://github.com/lebedov/scikit-cuda/issues/308 for CUDA 11
+        msg = "Testing 3D cufft with plan destruction & re-creation"
+
+        fft_ok = True
+        try:
+            import numpy as np
+            import pycuda.gpuarray as cua
+            import pycuda.tools as cu_tools
+            from pycuda.elementwise import ElementwiseKernel as CU_ElK
+            from pycuda.reduction import ReductionKernel as CU_RedK
+            import pycuda.driver as cu_drv
+            from pynx.processing_unit.cu_processing_unit import default_processing_unit as pu
+            import skcuda.fft as cu_fft
+            pu.init_cuda(test_fft=False, verbose=False)
+
+            fft_shape = (128, 128)
+            plan = pu.cu_fft_get_plan(fft_shape, batch=1, dtype_src=np.complex64, dtype_dest=np.complex64)
+            a = cua.to_gpu(np.random.uniform(0, 1, fft_shape).astype(np.complex64))
+            cu_fft.fft(a, a, plan)
+            tmp = cua.sum(a)
+
+            # This should be inhibited for CUDA 11
+            pu.cu_fft_free_plans()
+
+            plan = pu.cu_fft_get_plan(fft_shape, batch=1, dtype_src=np.complex64, dtype_dest=np.complex64)
+            cu_fft.fft(a, a, plan)
+            tmp = cua.sum(a)
+
+            cu_drv.Context.synchronize()
+        except:
+            msg += "\n" + traceback.format_exc()
+            fft_ok = False
+
+        self.assertTrue(fft_ok, msg=msg)
+
 
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     def test_cuda_device_available_gpu_speed(self):
