@@ -9,6 +9,7 @@ import sys
 import unittest
 import traceback
 import os
+
 exclude_cuda = False
 exclude_opencl = False
 if 'PYNX_PU' in os.environ:
@@ -87,7 +88,8 @@ class TestOpenCL(unittest.TestCase):
                         break
                 if opencl_gpu_device is not None:
                     break
-            cl_ctx = cl.Context([opencl_gpu_device])
+            from pynx.processing_unit.cl_processing_unit import CLProcessingUnit
+            cl_ctx = CLProcessingUnit.get_context(opencl_gpu_device)
             cl_queue = cl.CommandQueue(cl_ctx)
             cl_psi = cla.empty(cl_queue, (100,), np.float32)
             cl_psi.fill(np.float32(1))
@@ -116,7 +118,8 @@ class TestOpenCL(unittest.TestCase):
                         break
                 if opencl_gpu_device is not None:
                     break
-            cl_ctx = cl.Context([opencl_gpu_device])
+            from pynx.processing_unit.cl_processing_unit import CLProcessingUnit
+            cl_ctx = CLProcessingUnit.get_context(opencl_gpu_device)
             cl_queue = cl.CommandQueue(cl_ctx)
             cl_psi1 = cla.empty(cl_queue, (100,), np.float32)
             cl_psi2 = cla.empty(cl_queue, (100,), np.float32)
@@ -146,7 +149,8 @@ class TestOpenCL(unittest.TestCase):
                         break
                 if opencl_gpu_device is not None:
                     break
-            cl_ctx = cl.Context([opencl_gpu_device])
+            from pynx.processing_unit.cl_processing_unit import CLProcessingUnit
+            cl_ctx = CLProcessingUnit.get_context(opencl_gpu_device)
             cl_queue = cl.CommandQueue(cl_ctx)
             cl_psi = cla.empty(cl_queue, (128,), np.complex64)
             gpyfft_plan = gpyfft.FFT(cl_ctx, cl_queue, cl_psi, None)
@@ -177,7 +181,8 @@ class TestOpenCL(unittest.TestCase):
                         break
                 if opencl_gpu_device is not None:
                     break
-            cl_ctx = cl.Context([opencl_gpu_device])
+            from pynx.processing_unit.cl_processing_unit import CLProcessingUnit
+            cl_ctx = CLProcessingUnit.get_context(opencl_gpu_device)
             cl_queue = cl.CommandQueue(cl_ctx)
             cl_psi = cla.empty(cl_queue, (64, 64), np.complex64)
             gpyfft_plan = gpyfft.FFT(cl_ctx, cl_queue, cl_psi, None)
@@ -210,7 +215,8 @@ class TestOpenCL(unittest.TestCase):
                         break
                 if opencl_gpu_device is not None:
                     break
-            cl_ctx = cl.Context([opencl_gpu_device])
+            from pynx.processing_unit.cl_processing_unit import CLProcessingUnit
+            cl_ctx = CLProcessingUnit.get_context(opencl_gpu_device)
             cl_queue = cl.CommandQueue(cl_ctx)
             cl_psi = cla.empty(cl_queue, (16, 64, 64), np.complex64)
             cl_psi.fill(np.complex64(1))
@@ -241,7 +247,8 @@ class TestOpenCL(unittest.TestCase):
                         break
                 if opencl_gpu_device is not None:
                     break
-            cl_ctx = cl.Context([opencl_gpu_device])
+            from pynx.processing_unit.cl_processing_unit import CLProcessingUnit
+            cl_ctx = CLProcessingUnit.get_context(opencl_gpu_device)
             cl_queue = cl.CommandQueue(cl_ctx)
             cl_psi = cla.empty(cl_queue, (64, 64, 64), np.complex64)
             cl_psi.fill(np.complex64(1))
@@ -262,7 +269,7 @@ class TestOpenCL(unittest.TestCase):
         msg = "Testing pynx.processing_unit.opencl_device.available_gpu_speed()"
         try:
             from pynx.processing_unit.opencl_device import available_gpu_speed
-            tmp = available_gpu_speed(fft_shape=(16,32,32), verbose=False)
+            tmp = available_gpu_speed(fft_shape=(16, 32, 32), verbose=False)
             ok = True
         except:
             msg += "\n" + traceback.format_exc()
