@@ -383,7 +383,7 @@ class CLOperatorBraggPtycho(OperatorBraggPtycho):
             src = {'obj': pty._cl_obj, 'probe': pty._cl_probe, 'psi': pty._cl_psi, 'psi_v': pty._cl_psi_v}
         else:
             src = pty._cl_view[i_source]
-        if i_dest is 0:
+        if i_dest == 0:
             pty._cl_obj = cl.array.empty_like(src['obj'])
             pty._cl_probe = cl.array.empty_like(src['probe'])
             pty._cl_psi = cl.array.empty_like(src['psi'])
@@ -501,7 +501,7 @@ class FreePU(CLOperatorBraggPtycho):
                 pty.__delattr__(o, None)
         if has_attr_not_none(pty, "_cl_psi_v"):
             pty._cl_psi_v = {}
-        self.processing_unit.gpyfft_plan.data = None
+        self.processing_unit.cl_fft_free_plans()
         pty._cl_obs_v = None
         return pty
 
@@ -592,8 +592,8 @@ class FT(CLOperatorBraggPtycho):
         self.scale = scale
 
     def op(self, pty):
-        self.processing_unit.cl_fft_set_plan(pty._cl_psi, axes=None)
-        for e in self.processing_unit.gpyfft_plan.enqueue(forward=True):
+        plan = self.processing_unit.cl_fft_get_plan(pty._cl_psi, axes=None)
+        for e in plan.enqueue(forward=True):
             e.wait()  # Needed as CLFFT may use its own queues
         if self.scale:
             self.processing_unit.cl_scale(pty._cl_psi, np.float32((pty._cl_psi[0, 0, 0].size) ** (-1 / 3.)))
@@ -614,8 +614,8 @@ class IFT(CLOperatorBraggPtycho):
         self.scale = scale
 
     def op(self, pty):
-        self.processing_unit.cl_fft_set_plan(pty._cl_psi, axes=None)
-        for e in self.processing_unit.gpyfft_plan.enqueue(forward=False):
+        plan = self.processing_unit.cl_fft_get_plan(pty._cl_psi, axes=None)
+        for e in plan.enqueue(forward=False):
             e.wait()  # Needed as CLFFT may use its own queues
         if self.scale:
             self.processing_unit.cl_scale(pty._cl_psi, np.float32((pty._cl_psi[0, 0, 0].size) ** (1 / 3.)))
@@ -1284,8 +1284,8 @@ class _CGGamma(CLOperatorBraggPtycho):
                                                       pixel_size_object, f,
                                                       p._cl_obs_v[i].npsi, self.processing_unit.cl_stack_size,
                                                       nx, ny, nz, nxo, nyo, nzo, nb_obj, nb_probe)
-            self.processing_unit.cl_fft_set_plan(clpsi, axes=None)
-            for e in self.processing_unit.gpyfft_plan.enqueue(forward=True):
+            plan = self.processing_unit.cl_fft_get_plan(clpsi, axes=None)
+            for e in plan.enqueue(forward=True):
                 e.wait()
             # TODO: why is this scaling useful for convergence ? Numerical stability ?
             self.processing_unit.cl_scale(clpsi, np.float32((clpsi[0, 0, 0].size) ** (-1 / 3.)))
