@@ -6,8 +6,9 @@
 #         Vincent Favre-Nicolin, favre@esrf.fr
 
 
+import os
+import psutil
 import types
-import timeit
 import time
 from sys import stdout
 import warnings
@@ -15,6 +16,7 @@ import numpy as np
 from scipy.fftpack import fftn, ifftn, fftshift, fftfreq
 from scipy.spatial import cKDTree
 from scipy.interpolate import griddata
+from scipy.ndimage.measurements import center_of_mass
 from matplotlib.backends.backend_agg import FigureCanvasAgg
 from matplotlib.figure import Figure
 from pynx.utils.matplotlib import pyplot as plt
@@ -24,7 +26,9 @@ from .shape import get_view_coord
 from ..operator import has_attr_not_none, OperatorSum, OperatorPower, OperatorException
 from ..utils.math import ortho_modes
 from . import analysis
-from ..version import __version__
+from ..version import get_git_version
+
+_pynx_version = get_git_version()
 from ..wavefront import Wavefront, operator, UserWarningWavefrontNearFieldPropagation
 
 
@@ -154,7 +158,7 @@ class CPUOperatorPtycho(OperatorPtycho):
             src = {'obj': pty._obj, 'probe': pty._probe, 'psi': pty._psi, 'psi_v': pty._psi_v}
         else:
             src = pty._cpu_view[i_source]
-        if i_dest is 0:
+        if i_dest == 0:
             pty._obj = np.empty_like(src['obj'])
             pty._probe = np.empty_like(src['probe'])
             pty._psi = np.empty_like(src['psi'])
@@ -270,27 +274,36 @@ class ShowObjProbe(CPUOperatorPtycho):
     Class to display object and probe during an optimization
     """
 
-    def __init__(self, fig_num=-1, title=None):
+    def __init__(self, fig_num=-1, title=None, remove_obj_phase_ramp=True):
         """
 
         :param fig_num: the matplotlib figure number. if None, a new figure will be created. if -1 (the default), the
                         current figure will be used.
         :param title: the title for the view. If None, a default title will be used.
+        :param remove_obj_phase_ramp: if True, the object will be displayed after removing the phase
+                                      ramp coming from the imperfect centring of the diffraction data
+                                      (sub-pixel shift). Calculated diffraction patterns using such a
+                                      corrected object will present a sub-pixel shift relative to the
+                                      diffraction data. The ramp information comes from the PtychoData
+                                      phase_ramp_d{x,y} attributes, and are not re-calculated.
         """
         super(ShowObjProbe, self).__init__()
         self.fig_num = fig_num
         self.title = title
+        self.remove_obj_phase_ramp = remove_obj_phase_ramp
 
-    def op(self, p):
+    def op(self, p: Ptycho):
         if p.data.near_field:
             show_obj_probe(p.get_obj(), p.get_probe(), stit=self.title, fig_num=self.fig_num,
                            pixel_size_object=p.pixel_size_object, scan_area_obj=None, scan_area_probe=None,
-                           scan_pos=p.scan_area_points, minimize_obj_phase=False)
+                           scan_pos=p.get_scan_area_points())
         else:
-            show_obj_probe(p.get_obj(), p.get_probe(), stit=self.title, fig_num=self.fig_num,
+            obj = p.get_obj(remove_obj_phase_ramp=self.remove_obj_phase_ramp)
+
+            show_obj_probe(obj, p.get_probe(), stit=self.title, fig_num=self.fig_num,
                            pixel_size_object=p.pixel_size_object,
-                           scan_area_obj=p.scan_area_obj, scan_area_probe=p.scan_area_probe,
-                           scan_pos=p.scan_area_points)
+                           scan_area_obj=p.get_scan_area_obj(), scan_area_probe=p.get_scan_area_probe(),
+                           scan_pos=p.get_scan_area_points())
         return p
 
     def timestamp_increment(self, p):
@@ -316,6 +329,29 @@ class FreePU(CPUOperatorPtycho):
         return pty
 
 
+class MemUsage(CPUOperatorPtycho):
+    """
+    Print memory usage of current process (RSS on host)
+    """
+
+    def __init__(self, verbose=True):
+        super(MemUsage, self).__init__()
+        self.verbose = verbose
+
+    def op(self, p: Ptycho):
+        """
+
+        :param p: the ptycho object this operator applies to
+        :return: the updated ptycho object
+        """
+        # TODO: also print in detail array memory used.
+        process = psutil.Process(os.getpid())
+        rss = process.memory_info().rss
+
+        p.print("Mem Usage: RSS= %6.1f Mbytes (process)" % (rss / 1024 ** 2))
+        return p
+
+
 class Scale(CPUOperatorPtycho):
     """
     Multiply the ptycho object by a scalar (real or complex).
@@ -509,18 +545,28 @@ class Propagate(CPUOperatorPtycho):
 
 class Calc2Obs(CPUOperatorPtycho):
     """
-    Copy the calculated intensities to the observed ones. Can be used for simulation. Applies to a stack of N views,
-    assumes the current Psi is already in Fourier space.
+    Copy the calculated intensities to the observed ones, optionally with Poisson noise.
+    This operator will loop other all stacks of frames, multiply object and probe and
+    propagate the wavefront to the detector, and compute the new intensities.
     """
 
-    def __init__(self):
+    def __init__(self, nb_photons_per_frame=None, poisson_noise=False):
         """
 
         """
         super(Calc2Obs, self).__init__()
+        self.nb_photons_per_frame = nb_photons_per_frame
+        self.poisson_noise = poisson_noise
 
     def op(self, p):
         p.data.iobs = (np.abs(p._psi) ** 2).sum(axis=(0, 1))
+
+        if self.nb_photons_per_frame is not None:
+            p.data.iobs *= self.nb_photons_per_frame / (p.data.iobs.sum() / len(p.data.iobs))
+
+        if self.poisson_noise:
+            p.data.iobs = np.random.poisson(p.data.iobs).astype(np.float32)
+
         return p
 
 
@@ -667,7 +713,7 @@ class AP(CPUOperatorPtycho):
     """
 
     def __init__(self, update_object=True, update_probe=False, update_background=False, floating_intensity=False,
-                 nb_cycle=1, calc_llk=False, show_obj_probe=False, fig_num=-1):
+                 nb_cycle=1, calc_llk=False, show_obj_probe=False, fig_num=-1, zero_phase_ramp=True):
         """
 
         :param update_object: update object ?
@@ -680,6 +726,9 @@ class AP(CPUOperatorPtycho):
         :param show_obj_probe: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object and probe, as for ShowObjProbe()
+        :param zero_phase_ramp: if True, the conjugate phase ramp in the object and probe will be removed
+                                by centring the FT of the probe, at the end and before every display.
+                                Ignored for near field.
         """
         super(AP, self).__init__()
         self.update_object = update_object
@@ -690,6 +739,7 @@ class AP(CPUOperatorPtycho):
         self.calc_llk = calc_llk
         self.show_obj_probe = show_obj_probe
         self.fig_num = fig_num
+        self.zero_phase_ramp = zero_phase_ramp
 
     def __pow__(self, n):
         """
@@ -700,11 +750,10 @@ class AP(CPUOperatorPtycho):
         assert isinstance(n, int) or isinstance(n, np.integer)
         return AP(update_object=self.update_object, update_probe=self.update_probe,
                   floating_intensity=self.floating_intensity, nb_cycle=self.nb_cycle * n,
-                  calc_llk=self.calc_llk, show_obj_probe=self.show_obj_probe, fig_num=self.fig_num)
+                  calc_llk=self.calc_llk, show_obj_probe=self.show_obj_probe, fig_num=self.fig_num,
+                  zero_phase_ramp=self.zero_phase_ramp)
 
     def op(self, p: Ptycho):
-        t0 = timeit.default_timer()
-        ic_dt = 0
         for ic in range(self.nb_cycle):
             calc_llk = False
             if self.calc_llk:
@@ -717,13 +766,8 @@ class AP(CPUOperatorPtycho):
             p = Psi2ObjProbe(update_object=self.update_object, update_probe=self.update_probe) * p
 
             if calc_llk:
-                # Average time/cycle over the last N cycles
-                dt = (timeit.default_timer() - t0) / (ic - ic_dt + 1)
-                ic_dt = ic + 1
-                t0 = timeit.default_timer()
-
                 p.update_history(mode='llk', update_obj=self.update_object, update_probe=self.update_probe,
-                                 update_background=self.update_background, update_pos=False, dt=dt, algorithm='AP',
+                                 update_background=self.update_background, update_pos=False, algorithm='AP',
                                  verbose=True)
             else:
                 p.history.insert(p.cycle, update_obj=self.update_object, update_probe=self.update_probe,
@@ -734,9 +778,13 @@ class AP(CPUOperatorPtycho):
             if self.show_obj_probe:
                 if ic % self.show_obj_probe == 0 or ic == self.nb_cycle - 1:
                     s = algo_string('AP', p, self.update_object, self.update_probe, self.update_background)
-                    tit = "%s #%3d, LLKn(p)=%8.3f" % (s, ic, p.llk_poisson / p.nb_obs)
+                    tit = "%s #%3d, LLKn(p)=%8.3f" % (s, ic, p.get_llk('poisson'))
+                    if self.zero_phase_ramp:
+                        p = ZeroPhaseRamp(obj=True) * p
                     p = ShowObjProbe(fig_num=self.fig_num, title=tit) * p
 
+        if self.zero_phase_ramp and not self.show_obj_probe:
+            p = ZeroPhaseRamp(obj=True) * p
         return p
 
 
@@ -748,7 +796,7 @@ class DM(CPUOperatorPtycho):
 
     def __init__(self, update_object=True, update_probe=True, nb_cycle=1, calc_llk=False, show_obj_probe=False,
                  fig_num=-1, obj_smooth_sigma=0, obj_inertia=0.01, probe_smooth_sigma=0, probe_inertia=0.001,
-                 center_probe_n=0, center_probe_max_shift=5, loop_obj_probe=1):
+                 center_probe_n=0, center_probe_max_shift=5, loop_obj_probe=1, zero_phase_ramp=True):
         """
         TODO: Not all parameters are implemented in the CPU DM version.
         :param update_object: update object ?
@@ -759,6 +807,9 @@ class DM(CPUOperatorPtycho):
         :param show_obj_probe: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object and probe, as for ShowObjProbe()
+        :param zero_phase_ramp: if True, the conjugate phase ramp in the object and probe will be removed
+                                by centring the FT of the probe, at the end and before every display.
+                                Ignored for near field.
         """
         super(DM, self).__init__()
         self.nb_cycle = nb_cycle
@@ -767,6 +818,7 @@ class DM(CPUOperatorPtycho):
         self.calc_llk = calc_llk
         self.show_obj_probe = show_obj_probe
         self.fig_num = fig_num
+        self.zero_phase_ramp = zero_phase_ramp
 
     def __pow__(self, n):
         """
@@ -776,14 +828,13 @@ class DM(CPUOperatorPtycho):
         """
         assert isinstance(n, int) or isinstance(n, np.integer)
         return DM(update_object=self.update_object, update_probe=self.update_probe, nb_cycle=self.nb_cycle * n,
-                  calc_llk=self.calc_llk, show_obj_probe=self.show_obj_probe, fig_num=self.fig_num)
+                  calc_llk=self.calc_llk, show_obj_probe=self.show_obj_probe, fig_num=self.fig_num,
+                  zero_phase_ramp=self.zero_phase_ramp)
 
     def op(self, p):
         # Calculate starting Psi
         p = ObjProbe2Psi() * p
 
-        t0 = timeit.default_timer()
-        ic_dt = 0
         for ic in range(self.nb_cycle):
             calc_llk = False
             if self.calc_llk:
@@ -811,13 +862,8 @@ class DM(CPUOperatorPtycho):
                 else:
                     p = LLK() * FT() * ObjProbe2Psi() * p
 
-                # Average time/cycle over the last N cycles
-                dt = (timeit.default_timer() - t0) / (ic - ic_dt + 1)
-                ic_dt = ic + 1
-                t0 = timeit.default_timer()
-
                 p.update_history(mode='llk', update_obj=self.update_object, update_probe=self.update_probe,
-                                 update_background=False, update_pos=False, dt=dt, algorithm='DM',
+                                 update_background=False, update_pos=False, algorithm='DM',
                                  verbose=True)
                 # Restore correct Psi
                 p._psi = psi1
@@ -829,10 +875,14 @@ class DM(CPUOperatorPtycho):
             if self.show_obj_probe:
                 if ic % self.show_obj_probe == 0 or ic == self.nb_cycle - 1:
                     s = algo_string('DM', p, self.update_object, self.update_probe)
-                    tit = "%s #%3d, LLKn(p)=%8.3f" % (s, ic, p.llk_poisson / p.nb_obs)
+                    tit = "%s #%3d, LLKn(p)=%8.3f" % (s, ic, p.get_llk('poisson'))
+                    if self.zero_phase_ramp:
+                        p = ZeroPhaseRamp(obj=False) * p
                     p = ShowObjProbe(fig_num=self.fig_num, title=tit) * p
             p.cycle += 1
 
+        if self.zero_phase_ramp and not self.show_obj_probe:
+            p = ZeroPhaseRamp(obj=False) * p
         return p
 
 
@@ -842,7 +892,8 @@ class ML(CPUOperatorPtycho):
     """
 
     def __init__(self, nb_cycle=1, update_object=True, update_probe=False, update_background=False,
-                 reg_fac_obj=0, reg_fac_probe=0, calc_llk=False, show_obj_probe=False, fig_num=-1):
+                 reg_fac_obj=0, reg_fac_probe=0, calc_llk=False, show_obj_probe=False, fig_num=-1,
+                 zero_phase_ramp=True):
         """
 
         :param update_object: update object ?
@@ -855,6 +906,8 @@ class ML(CPUOperatorPtycho):
         :param show_obj_probe: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object and probe, as for ShowObjProbe()
+        :param zero_phase_ramp: if True, the conjugate phase ramp in the object and probe will be removed
+                                by centring the FT of the probe, at the beginning and end. Ignored for near field.
         """
         super(ML, self).__init__()
         self.nb_cycle = nb_cycle
@@ -866,6 +919,7 @@ class ML(CPUOperatorPtycho):
         self.calc_llk = calc_llk
         self.show_obj_probe = show_obj_probe
         self.fig_num = fig_num
+        self.zero_phase_ramp = zero_phase_ramp
 
     def __pow__(self, n):
         """
@@ -877,9 +931,12 @@ class ML(CPUOperatorPtycho):
         return ML(nb_cycle=self.nb_cycle * n, update_object=self.update_object, update_probe=self.update_probe,
                   update_background=self.update_background, reg_fac_obj=self.reg_fac_obj,
                   reg_fac_probe=self.reg_fac_probe, calc_llk=self.calc_llk, show_obj_probe=self.show_obj_probe,
-                  fig_num=self.fig_num)
+                  fig_num=self.fig_num, zero_phase_ramp=self.zero_phase_ramp)
 
     def op(self, p):
+        # First perform an AP cycle to make sure object and probe are properly scaled with respect to iobs
+        p = AP(update_object=self.update_object, update_probe=self.update_probe,
+               update_background=self.update_background, zero_phase_ramp=self.zero_phase_ramp) * p
         ny = np.int32(p._probe.shape[-2])
         nx = np.int32(p._probe.shape[-1])
         nb_probe = np.int32(p._probe.shape[0])
@@ -905,8 +962,6 @@ class ML(CPUOperatorPtycho):
         if self.update_background:
             background_grad = np.empty_like(p._background)
 
-        t0 = timeit.default_timer()
-        ic_dt = 0
         for ic in range(self.nb_cycle):
             calc_llk = False
             if self.calc_llk:
@@ -1064,13 +1119,9 @@ class ML(CPUOperatorPtycho):
 
             if calc_llk:
                 p = LLK() * Propagate(forward=True) * ObjProbe2Psi() * p
-                # Average time/cycle over the last N cycles
-                dt = (timeit.default_timer() - t0) / (ic - ic_dt + 1)
-                ic_dt = ic + 1
-                t0 = timeit.default_timer()
 
                 p.update_history(mode='llk', update_obj=self.update_object, update_probe=self.update_probe,
-                                 update_background=self.update_background, update_pos=False, dt=dt, algorithm='ML',
+                                 update_background=self.update_background, update_pos=False, algorithm='ML',
                                  verbose=True)
             else:
                 p.history.insert(p.cycle, update_obj=self.update_object, update_probe=self.update_probe,
@@ -1080,10 +1131,12 @@ class ML(CPUOperatorPtycho):
             if self.show_obj_probe:
                 if ic % self.show_obj_probe == 0 or ic == self.nb_cycle - 1:
                     s = algo_string('ML', p, self.update_object, self.update_probe, self.update_background)
-                    tit = "%s #%3d, LLKn(p)=%8.3f" % (s, ic, p.llk_poisson / p.nb_obs)
+                    tit = "%s #%3d, LLKn(p)=%8.3f" % (s, ic, p.get_llk('poisson'))
                     p = ShowObjProbe(fig_num=self.fig_num, title=tit) * p
             p.cycle += 1
 
+        if self.zero_phase_ramp:
+            p = ZeroPhaseRamp(obj=True) * p
         return p
 
 
@@ -1111,7 +1164,98 @@ class ScaleObjProbe(CPUOperatorPtycho):
         p._probe *= np.sqrt(os / ps * s)
         p._obj *= np.sqrt(ps / os * s)
         if self.verbose:
-            print("ScaleObjProbe:", ps, os, s, np.sqrt(os / ps * s), np.sqrt(ps / os * s))
+            p.print("ScaleObjProbe:", ps, os, s, np.sqrt(os / ps * s), np.sqrt(ps / os * s))
+        return p
+
+
+class ApplyPhaseRamp(CPUOperatorPtycho):
+    """
+    Apply a given phase ramp to the object and/or probe. The actual phase factor is:
+    For the probe: np.exp(-2j*np.pi*(x * dx + y*dy))
+    For the object: np.exp(2j*np.pi*(x * dx + y*dy))
+    Where (x,y) are reduced pixel coordinates going from -.5 to .5 for the probe,
+    and  +/-0.5 * nxo/nx (or nyo/ny) for the object.
+
+    If the phase ramp is applied to both object and probe, the calculated
+    intensity patterns remain unchanged.
+    """
+
+    def __init__(self, dx, dy, probe=False, obj=False):
+        """
+
+        :param dx, dy: relative shifts from the centre, calculated in reciprocal space
+                       (probe array pixel coordinates)
+        :param obj: if True, apply the correction to the object.
+        :param probe: if True, apply the correction to the probe.
+        """
+        super(ApplyPhaseRamp, self).__init__()
+        self.probe = probe
+        self.obj = obj
+        self.dx = np.float32(dx)
+        self.dy = np.float32(dy)
+
+    def op(self, p: Ptycho):
+        nz, ny, nx = np.int32(p._probe.shape[0]), np.int32(p._probe.shape[1]), np.int32(p._probe.shape[2])
+        nyo, nxo = np.int32(p._obj.shape[1]), np.int32(p._obj.shape[2])
+
+        # Corr ramp with opposite signs for object and probe
+        if self.probe:
+            y, x = np.meshgrid(fftshift(fftfreq(ny)), fftshift(fftfreq(nx)), indexing='ij')
+            p._probe *= np.exp(-2j * np.pi * (x * self.dx + y * self.dy))
+        if self.obj:
+            y, x = np.meshgrid(fftshift(fftfreq(nyo, d=ny / nyo)).astype(np.float32),
+                               fftshift(fftfreq(nxo, d=nx / nxo)).astype(np.float32), indexing='ij')
+            p._obj *= np.exp(2j * np.pi * (x * self.dx + y * self.dy))
+        return p
+
+
+class ZeroPhaseRamp(CPUOperatorPtycho):
+    """
+    Operator to remove the linear phase ramp on both the object and the probe, for far field ptycho.
+    This first computes the center of mass of the square norm of Fourier transform of the probe,
+    and then corrects both probe and object for the phase ramp corresponding to the shift
+    relative to the center of the array.
+
+    Then, a remaining phase ramp in the object can optionally be computed by:
+    - computing the sum of the calculated intensity along all frames
+    - computing the shift of the center of mass of that intensity w/r to the array center
+    - the phase ramp parameters are stored in the ptycho object, but not applied as the calculated
+    """
+
+    def __init__(self, obj=False):
+        """
+
+        :param obj: if True, after correcting both object and probe from the probe phase ramp,
+                    the object phase ramp is evaluated from the center of the calculated diffraction.
+        :param dx, dy: probe shifts to correct for. Should be None, and will be calculated, but
+                       can be supplied for manual correction
+        """
+        super(ZeroPhaseRamp, self).__init__()
+        self.obj = obj
+
+    def op(self, p: Ptycho):
+        if p.data.near_field:
+            return p
+        ny, nx = np.int32(p._probe.shape[1]), np.int32(p._probe.shape[2])
+
+        cyx = center_of_mass((abs(fftshift(fftn(fftshift(p._probe)))) ** 2).sum(axis=0))
+        dx, dy = cyx[1] - nx / 2, cyx[0] - ny / 2
+        # print("ZeroPhaseRamp(): (dx, dy)[probe] = (%6.3f, %6.3f)" % (dx, dy))
+
+        p = ApplyPhaseRamp(dx, dy, obj=True, probe=True) * p
+
+        if self.obj:
+            # Compute the shift of the calculated frame to determine the object ramp
+            p = Propagate() * ObjProbe2Psi() * p
+            icalc_sum = (abs(p._psi) ** 2).sum(axis=(0, 1, 2))
+
+            # Compute shift of center of mass
+            cyx = center_of_mass(fftshift(icalc_sum))
+            dx, dy = cyx[1] - nx / 2, cyx[0] - ny / 2
+            # print("ZeroPhaseRamp(): (dx, dy)[obj] = (%6.3f, %6.3f)[calc]" % (dx, dy))
+            p.data.phase_ramp_dx = dx
+            p.data.phase_ramp_dy = dy
+
         return p
 
 
@@ -1166,7 +1310,7 @@ class AnalyseProbe(CPUOperatorPtycho):
         self.save_prefix = save_prefix
 
     def op(self, p: Ptycho):
-        # Analyze probe modes
+        # Analyse probe modes
         probe = p.get_probe()
         if probe.shape[0] > 1 and self.modes:
             if self.show_plot or (self.save_prefix is not None):
@@ -1180,7 +1324,7 @@ class AnalyseProbe(CPUOperatorPtycho):
             if self.save_prefix is not None or self.show_plot:
                 dy = (6 + 1) / 72 / fig.get_size_inches()[1]
                 fig.text(dy / 5, dy / 2,
-                         "PyNX v%s, %s" % (__version__, time.strftime("%Y/%m/%d %H:%M:%S")),
+                         "PyNX v%s, %s" % (_pynx_version, time.strftime("%Y/%m/%d %H:%M:%S")),
                          fontsize=6, horizontalalignment='left', stretch='condensed')
                 canvas = FigureCanvasAgg(fig)
                 if self.save_prefix is not None:
@@ -1189,8 +1333,8 @@ class AnalyseProbe(CPUOperatorPtycho):
                     print("Saving probe modes to:%s" % (filename))
                 if self.show_plot:
                     try:
-                        # Not supported on all backends (e.g. nbagg)
                         plt.draw()
+                        plt.gcf().canvas.draw()
                         plt.pause(.001)
                     except:
                         pass
@@ -1198,7 +1342,7 @@ class AnalyseProbe(CPUOperatorPtycho):
         else:
             probe = probe[0]
         if self.focus:
-            # Analyze probe propagation of main mode
+            # Analyse probe propagation of main mode
             with warnings.catch_warnings() as w:
                 warnings.simplefilter('ignore', UserWarningWavefrontNearFieldPropagation)
                 p3d, vdz, izmax = analysis.probe_propagate(probe, (-2e-3, 2e-3, 200), p.data.pixel_size_object()[0],
@@ -1223,7 +1367,7 @@ class AnalyseProbe(CPUOperatorPtycho):
             if self.save_prefix is not None or self.show_plot:
                 dy = (6 + 1) / 72 / fig.get_size_inches()[1]
                 fig.text(dy / 5, dy / 2,
-                         "PyNX v%s, %s" % (__version__, time.strftime("%Y/%m/%d %H:%M:%S")),
+                         "PyNX v%s, %s" % (_pynx_version, time.strftime("%Y/%m/%d %H:%M:%S")),
                          fontsize=6, horizontalalignment='left', stretch='condensed')
                 if self.save_prefix is not None:
                     canvas = FigureCanvasAgg(fig)
@@ -1232,8 +1376,8 @@ class AnalyseProbe(CPUOperatorPtycho):
                     print("Saving propagated probe plot to:%s" % (filename))
                 if self.show_plot:
                     try:
-                        # Not supported on all backends (e.g. nbagg)
                         plt.draw()
+                        plt.gcf().canvas.draw()
                         plt.pause(.001)
                     except:
                         pass
@@ -1261,11 +1405,14 @@ class PlotPositions(CPUOperatorPtycho):
         self.fig_size = fig_size
 
     def op(self, p: Ptycho):
+        return self.plot(p.data.posx + p.data.posx_c, p.data.posy + p.data.posy_c,
+                         p.data.posx0 + p.data.posx_c, p.data.posy0 + p.data.posy_c, p)
+
+    def plot(self, x, y, x0, y0, p: Ptycho):
         if self.show_plot is False and self.save_prefix is None:
             # Dumb !
             return p
-        x, y = p.data.posx, p.data.posy
-        dx, dy = x - p.data.posx0, y - p.data.posy0
+        dx, dy = x - x0, y - y0
         r0 = np.sqrt(((x.max()) - x.min()) ** 2 + ((y.max()) - y.min()) ** 2)
 
         if np.allclose(dx / r0, 0) and np.allclose(dy / r0, 0):
@@ -1316,7 +1463,7 @@ class PlotPositions(CPUOperatorPtycho):
             fig = Figure(figsize=self.fig_size)
         ax = fig.add_subplot(121)
         figdy = (6 + 1) / 72 / fig.get_size_inches()[1]
-        ax.quiver(x * s, y * s, dx * s, dy * s, scale=1 / scale, angles='xy', scale_units='xy')
+        ax.quiver(x0 * s, y0 * s, dx * s, dy * s, scale=1 / scale, angles='xy', scale_units='xy')
 
         ax.set_xlabel(u"x(%s)" % unit_name, horizontalalignment='left')
         ax.set_ylabel(u"y(%s)" % unit_name)
@@ -1330,12 +1477,12 @@ class PlotPositions(CPUOperatorPtycho):
         ax = fig.add_subplot(122)
         # Create a heatmap of displacements
         # plt.quiver(x * s, y * s, dx * s, dy * s, scale=1 / scale, angles='xy', scale_units='xy')
-        ax.scatter(x * s, y * s, 1, color='k')
+        ax.scatter(x0 * s, y0 * s, 1, color='k')
         x1, x2 = ax.get_xlim()
         y1, y2 = ax.get_ylim()
         grid_y, grid_x = np.mgrid[y1 / s:y2 / s:400j, x1 / s:x2 / s:400j]
-        grid_ph = griddata((x, y), np.arctan2(dy, dx), (grid_x, grid_y), method='nearest')
-        grid_ampl = griddata((x, y), dr, (grid_x, grid_y), method='cubic', fill_value=0)
+        grid_ph = griddata((x0, y0), np.arctan2(dy, dx), (grid_x, grid_y), method='nearest')
+        grid_ampl = griddata((x0, y0), dr, (grid_x, grid_y), method='cubic', fill_value=0)
         rgba = complex2rgbalin(np.ma.masked_array(grid_ampl * np.exp(1j * grid_ph), mask=np.isnan(grid_ampl)))
         ax.imshow(rgba, origin='lower', extent=(x1, x2, y1, y2))
         ax.set_xlim(x1, x2)
@@ -1343,7 +1490,7 @@ class PlotPositions(CPUOperatorPtycho):
         ax = fig.add_axes((0.90, 0.20, 0.08, .08), facecolor='w')  # [left, bottom, width, height]
         colorwheel(ax=ax)
 
-        fig.text(figdy / 5, figdy / 2, "PyNX v%s, %s" % (__version__, time.strftime("%Y/%m/%d %H:%M:%S")),
+        fig.text(figdy / 5, figdy / 2, "PyNX v%s, %s" % (_pynx_version, time.strftime("%Y/%m/%d %H:%M:%S")),
                  fontsize=6, horizontalalignment='left', stretch='condensed')
         if self.save_prefix is not None:
             canvas = FigureCanvasAgg(fig)
@@ -1352,14 +1499,36 @@ class PlotPositions(CPUOperatorPtycho):
             print("Saving positions plot to:%s" % filename)
         if self.show_plot:
             try:
-                # Not supported on all backends (e.g. nbagg)
                 plt.draw()
+                plt.gcf().canvas.draw()
                 plt.pause(.001)
             except:
                 pass
 
         return p
 
+    def timestamp_increment(self, p):
+        # This operator does not alter the ptycho object or GPU arrays
+        pass
+
+
+class CalcIllumination(CPUOperatorPtycho):
+    """Compute the integrated illumination of the object by all probe positions
+    """
+
+    def op(self, p: Ptycho):
+        pr = p._probe
+        nprobe, ny, nx = pr.shape
+        nyo, nxo = p._obj.shape[-2:]
+        px, py = p.data.pixel_size_object()
+        obj_illumination = np.zeros_like(p._obj[0], dtype=np.float32)
+        for i in range(len(p.data.iobs)):
+            dy, dx = p.data.posy[i] / py, p.data.posx[i] / px
+            cx, cy = get_view_coord((nyo, nxo), (ny, nx), dx, dy, integer=True)
+            obj_illumination[cy:cy + ny, cx:cx + nx] += (abs(pr) ** 2).sum(axis=0)
+        p._obj_illumination = obj_illumination
+        return p
+
 
 class SelectStack(OperatorPtycho):
     """
