@@ -740,7 +740,7 @@ class Ptycho:
             p, g, e, nph = d['poisson'], d['gaussian'], d['euclidian'], d['nb_photons_calc']
             algo = ''
             if self.history_llk_t0_cycle is not None:
-                dt = (timeit.default_timer() - self.history_llk_t0) / (self.cycle-self.history_llk_t0_cycle)
+                dt = (timeit.default_timer() - self.history_llk_t0) / (self.cycle - self.history_llk_t0_cycle)
                 kwargs['dt'] = dt
             else:
                 dt = 0
@@ -1006,13 +1006,16 @@ def save_obj_probe_cxi(filename, obj, probe, wavelength, detector_distance, pixe
     result_1['title'] = 'Object'
     result_1.attrs['NX_class'] = 'NXdata'
     # Save the object as a Virtual Dataset, so that we can show scan_area and illumination
-    # as auxilliary signals. This requires hdf5>=1.10
+    # as auxiliary signals. This requires hdf5>=1.10
     obj_layout = h5py.VirtualLayout(shape=obj.shape, dtype=obj.dtype)
     aux = []
     for i in range(len(obj)):
         ds = result_1.create_dataset("Object-layer%02d" % i, data=obj[i], chunks=True, shuffle=True, compression="gzip")
         ds.attrs['interpretation'] = 'image'
-        obj_layout[i] = h5py.VirtualSource(ds)
+        # Explicitly using a link to the same file to avoid linking issue
+        # See https://github.com/h5py/h5py/issues/1546 fixed in h5py>=3.0
+        # obj_layout[i] = h5py.VirtualSource(ds)
+        obj_layout[i] = h5py.VirtualSource(".", ds.name, ds.shape, ds.dtype)
         if i > 0:
             aux.append("Object-layer%02d" % i)
     result_1.create_virtual_dataset("data", obj_layout)
