@@ -23,20 +23,7 @@ except:
 import h5py
 import numpy as np
 
-try:
-    import silx
-
-    if silx.version_info.minor < 3:
-        # start using silx with 0.3-dev
-        silx = None
-    else:
-        from silx.io.specfile import SpecFile
-    print("Using SILX for spec access!")
-except ImportError:
-    silx = None
-
-if silx is None:
-    from Vincent import spec
+import silx
 
 from .runner import PtychoRunner, PtychoRunnerScan, PtychoRunnerException, params_generic
 
@@ -83,10 +70,10 @@ for k, v in params_beamline.items():
 
 
 class PtychoRunnerScanCristal(PtychoRunnerScan):
-    def __init__(self, params, scan):
-        super(PtychoRunnerScanCristal, self).__init__(params, scan)
+    def __init__(self, params, scan, timings=None):
+        super(PtychoRunnerScanCristal, self).__init__(params, scan, timings=timings)
 
-    def load_data(self):
+    def load_scan(self):
         h5 = h5py.File(self.params['data'], 'r')
 
         xmot, ymot = self.params['ptychomotors'].split(',')[0:2]
@@ -124,13 +111,17 @@ class PtychoRunnerScanCristal(PtychoRunnerScan):
                 imgn = imgn[:N]
                 self.x = self.x[:N]
                 self.y = self.y[:N]
+        h5.close()
+        self.imgn = imgn
 
+    def load_data(self):
+        h5 = h5py.File(self.params['data'], 'r')
         # Load all frames
+        imgn = self.imgn
         t0 = timeit.default_timer()
         vimg = None
         d0 = 0
 
-        # TODO: use parallel process to load files and import frames ? (not possible using hdf5 ?)
         sys.stdout.write('Reading HDF5 frames: ')
         # frames are grouped in different subentries
         i0 = 0
@@ -152,17 +143,6 @@ class PtychoRunnerScanCristal(PtychoRunnerScan):
                 print("\nReading h5 data entry: %s" % (h5entry))
                 h5d = np.array(h5[h5entry][()])
             frame = h5d[i - i0]
-            # # Values of 2**32-1 and -2 are invalid (module gaps or invalidated pixels)
-            # # Need to test on all frames, as 'invalid' pixels may differ from frame to frame...
-            # tmp = (frame > 2 ** 32 - 3)
-            # if tmp.sum() > 0:
-            #     if self.raw_mask is None:
-            #         self.raw_mask = tmp.astype(np.int8)
-            #     else:
-            #         self.raw_mask[tmp > 0] = 1
-            # Set to zero masked pixels ? No, this will be done in center_crop_data()
-            # if self.raw_mask is not None:
-            #    frame *= (self.raw_mask == 0)
             if vimg is None:
                 vimg = np.empty((len(imgn), frame.shape[0], frame.shape[1]), dtype=frame.dtype)
             vimg[ii] = frame
