@@ -49,35 +49,14 @@ for k, v in params_beamline.items():
 
 
 class PtychoRunnerScanPtyPy(PtychoRunnerScan):
-    def __init__(self, params, scan):
-        super(PtychoRunnerScanPtyPy, self).__init__(params, scan)
+    def __init__(self, params, scan, timings=None):
+        super(PtychoRunnerScanPtyPy, self).__init__(params, scan, timings=timings)
 
-    def load_data(self):
+    def load_scan(self):
         h5data = h5py.File(self.params['data'], 'r')
-        self.params['instrument'] = self.params['data']
-        self.params['nrj'] = h5data['/info/energy'][()]  # Energy in keV
-        self.params['detectordistance'] = h5data['/info/distance'][()]
-
-        self.params['pixelsize'] = h5data['/info/psize'][()]
-        if np.isscalar(h5data['/info/rebin'][()]):
-            self.params['pixelsize'] *= h5data['/info/rebin'][()]
-
         # TODO: handle multiple chunks
         tmp = h5data['/chunks/0/positions'][()]
 
-        if False:
-            #OK 0.501
-            self.y, self.x = -tmp[:, 0], -tmp[:, 1]
-            do_transpose, do_flipud, do_fliplr = True, False, False
-        if False:
-            #OK 0.536
-            self.y, self.x = tmp[:, 0], -tmp[:, 1]
-            do_transpose, do_flipud, do_fliplr = True, True, False
-        if False:
-            #OK 0.604 - gives the same orientation as ptypy
-            self.y, self.x = tmp[:, 0], tmp[:, 1]
-            do_transpose, do_flipud, do_fliplr = True, True, True
-
         if True:
             self.x, self.y = tmp[:, 0], tmp[:, 1]
             do_transpose, do_flipud, do_fliplr = False, False, False
@@ -101,6 +80,18 @@ class PtychoRunnerScanPtyPy(PtychoRunnerScan):
                 imgn = imgn[:N]
                 self.x = self.x[:N]
                 self.y = self.y[:N]
+        self.imgn = imgn
+
+    def load_data(self):
+        imgn = self.imgn
+        h5data = h5py.File(self.params['data'], 'r')
+        self.params['instrument'] = self.params['data']
+        self.params['nrj'] = h5data['/info/energy'][()]  # Energy in keV
+        self.params['detectordistance'] = h5data['/info/distance'][()]
+
+        self.params['pixelsize'] = h5data['/info/psize'][()]
+        if np.isscalar(h5data['/info/rebin'][()]):
+            self.params['pixelsize'] *= h5data['/info/rebin'][()]
 
         # Load all frames
         print("Reading frames from PtyPy file:")
