@@ -15,28 +15,13 @@ import timeit
 
 try:
     import hdf5plugin
-
-    print("WARNING: hdf5plugin not found. This may prevent reading Eiger hdf5 files. Hint: 'pip3 install hdf5plugin'")
 except:
-    pass
+    print("WARNING: hdf5plugin not found. This may prevent reading Eiger hdf5 files. Hint: 'pip3 install hdf5plugin'")
 import fabio
 import h5py
 import numpy as np
 
-try:
-    import silx
-
-    if silx.version_info.minor < 3:
-        # start using silx with 0.3-dev
-        silx = None
-    else:
-        from silx.io.specfile import SpecFile
-    print("Using SILX for spec access!")
-except ImportError:
-    silx = None
-
-if silx is None:
-    from Vincent import spec
+import silx
 
 from .runner import PtychoRunner, PtychoRunnerScan, PtychoRunnerException, params_generic
 
@@ -44,17 +29,19 @@ helptext_beamline = """
 Script to perform a ptychography analysis on data from 25A@TPS
 
 Example:
-    pynx-tps25apty.py h5file=pty_41_61_master.h5 specfile=scan.dat scan=41 detectordistance=1.3
-      ptychomotors=pix,piz,-x,y probe=60e-6x200e-6,0.09 
+    pynx-tps25apty.py h5file=pty_41_61_master.h5 specfile=scan.dat probe=gauss,3e-6x3e-6
       algorithm=analysis,ML**100,DM**200,nbprobe=3,probe=1 verbose=10 save=all saveplot liveplot
 
 Command-line arguments (beamline-specific):
     scanfile=/some/dir/to/scan.txt: path to text file with motor positions [mandatory]
 
-    data=path/to/data.h5: path to hdf5 data, with images, wavelength, detector distance [mandatory]
+    data=path/to/data.h5: path to hdf5 data file, with images, wavelength, detector distance [mandatory]
+    
+    h5data=entry/data/data_%06d: relative path to data inside hdf5 file [default=entry/data/data_%06d]
 """
 
-params_beamline = {'scanfile': None, 'data': None, 'object': 'random,0.9,1,-.2,.2', 'instrument': 'TPS 25A'}
+params_beamline = {'scanfile': None, 'data': None, 'object': 'random,0.9,1,-.2,.2', 'instrument': 'TPS 25A',
+                   'h5data': 'entry/data/data_%06d'}
 
 params = params_generic.copy()
 for k, v in params_beamline.items():
@@ -62,13 +49,32 @@ for k, v in params_beamline.items():
 
 
 class PtychoRunnerScanTPS25A(PtychoRunnerScan):
-    def __init__(self, params, scan):
-        super(PtychoRunnerScanTPS25A, self).__init__(params, scan)
+    def __init__(self, params, scan, timings=None):
+        super(PtychoRunnerScanTPS25A, self).__init__(params, scan, timings=timings)
 
-    def load_data(self):
+    def load_scan(self):
         print("reading motor positions from: ", self.params['scanfile'])
         self.x, self.y = np.loadtxt('scan_2670.txt', delimiter=',', skiprows=1, unpack=True, usecols=(1, 2))
 
+        self.imgn = np.arange(len(self.x), dtype=np.int)
+
+        read_all_frames = True
+        if self.params['moduloframe'] is not None:
+            n1, n2 = self.params['moduloframe']
+            idx = np.where(self.imgn % n1 == n2)[0]
+            self.imgn = self.imgn.take(idx)
+            self.x = self.x.take(idx)
+            self.y = self.y.take(idx)
+
+        if self.params['maxframe'] is not None:
+            N = self.params['maxframe']
+            if len(self.imgn) > N:
+                print("MAXFRAME: only using first %d frames" % (N))
+                self.imgn = self.imgn[:N]
+                self.x = self.x[:N]
+                self.y = self.y[:N]
+
+    def load_data(self):
         self.h5 = h5py.File(self.params['data'], 'r')
 
         if self.params['pixelsize'] is None:
@@ -87,85 +93,49 @@ class PtychoRunnerScanTPS25A(PtychoRunnerScan):
         self.x *= 1e-6
         self.y *= 1e-6
 
-        imgn = np.arange(len(self.x), dtype=np.int)
-
-        read_all_frames = True
-        if self.params['moduloframe'] is not None:
-            n1, n2 = self.params['moduloframe']
-            idx = np.where(imgn % n1 == n2)[0]
-            if len(idx[0]) < len(imgn):
-                read_all_frames = False
-            imgn = imgn.take(idx)
-            self.x = self.x.take(idx)
-            self.y = self.y.take(idx)
-
-        if self.params['maxframe'] is not None:
-            N = self.params['maxframe']
-            if len(imgn) > N:
-                print("MAXFRAME: only using first %d frames" % (N))
-                imgn = imgn[:N]
-                self.x = self.x[:N]
-                self.y = self.y[:N]
-                read_all_frames = False
-
         # Load all frames
         t0 = timeit.default_timer()
-        vimg = None
-        d0 = 0
-        sys.stdout.write('Reading HDF5 frames: ')
+
+        self.print('Reading HDF5 frames: ')
         # frames are grouped in different subentries
-        i0 = 0
         entry0 = 1
-        h5_data_prefix = 'entry/data/data_%06d'
-        h5entry = h5_data_prefix % entry0
-        print("\nReading h5 data entry: %s" % (h5entry))
-        if read_all_frames:
-            # Read all the data frames in one go - faster
-            h5d = np.array(self.h5[h5entry][()])
-        else:
-            # Read frame-by-frame - more efficient, at least memory-wise
+        ct = 0
+        i0 = 0
+        vimg = None
+        # NB: it may be faster/easier to create a virtual dataset ?
+        while True:
+            h5entry = self.params["h5data"] % entry0
+            if h5entry not in self.h5:
+                break
             h5d = self.h5[h5entry]
-        ii = 0
-        for i in imgn:
-            if (i - imgn[0]) % 20 == 0:
-                sys.stdout.write('%d ' % (i - imgn[0]))
-                sys.stdout.flush()
-            # Load all frames
-            if i >= (i0 + len(h5d)):
-                # Open next data pack (read frame-by frame)
-                i0 += len(h5d)
-                entry0 += 1
-                h5entry = h5_data_prefix % entry0
-                print("\nReading h5 data entry: %s" % (h5entry))
-                if read_all_frames:
-                    # Read all the data frames in one go - faster
-                    h5d = np.array(self.h5[h5entry][()])
-                else:
-                    # Read frame-by-frame - more efficient, at least memory-wise
-                    h5d = self.h5[h5entry]
-            frame = h5d[i - i0]
-            # Values of 2**32-1 and -2 are invalid (module gaps or invalidated pixels)
-            # Need to test on all frames, as 'invalid' pixels may differ from frame to frame...
-            tmp = (frame > 2 ** 32 - 3)
-            if tmp.sum() > 0:
-                if self.raw_mask is None:
-                    self.raw_mask = tmp.astype(np.int8)
-                else:
-                    self.raw_mask[tmp > 0] = 1
-            # Set to zero masked pixels ? No, this will be done in center_crop_data()
-            # if self.raw_mask is not None:
-            #    frame *= (self.raw_mask == 0)
-            if vimg is None:
-                vimg = np.empty((len(imgn), frame.shape[0], frame.shape[1]), dtype=frame.dtype)
-            vimg[ii] = frame
-            d0 += frame
-            ii += 1
-        print("\n")
+            nb = len(h5d)
+
+            # Read all suitable frames
+            idx = np.where(np.logical_and(self.imgn >= i0, self.imgn < (i0 + nb)))[0]
+            if len(idx):
+                self.print("Reading h5 data entry: %s [%d frames]" % (h5entry, len(idx)))
+                if vimg is None:
+                    # Init array
+                    frame = np.array(h5d[0])
+                    vimg = np.empty((len(self.imgn), frame.shape[0], frame.shape[1]), dtype=frame.dtype)
+                vimg[idx] = h5d[self.imgn[idx] - i0]
+                ct += len(idx)
+
+            entry0 += 1
+            i0 += nb
+        if ct != len(self.imgn):
+            raise PtychoRunnerException("Could not read the expected number of frames: %d < %d" % (ct, len(self.imgn)))
+
         dt = timeit.default_timer() - t0
-        print('Time to read all frames: %4.1fs [%5.2f Mpixel/s]' % (dt, d0.size * len(vimg) / 1e6 / dt))
-        if self.raw_mask is not None:
-            if self.raw_mask.sum() > 0:
-                print("\nMASKING %d pixels from detector flags" % (self.raw_mask.sum()))
+        print('Time to read all frames: %4.1fs [%5.2f Mpixel/s]' % (dt, vimg.size / 1e6 / dt))
+
+        # Build mask. Values of 2**32-1 and -2 are invalid (module gaps or invalidated pixels)
+        # TODO: mask independently the different frames
+        self.raw_mask = ((vimg > (2 ** 32 - 3)).sum(axis=0) > 0).astype(np.int8)
+        if self.raw_mask.sum() == 0:
+            self.raw_mask = None
+        else:
+            self.print("\nMASKING %d pixels from detector flags" % (self.raw_mask.sum()))
 
         self.raw_data = vimg
         self.load_data_post_process()
@@ -189,7 +159,7 @@ class PtychoRunnerTPS25A(PtychoRunner):
         Returns:
             True if the argument is interpreted, false otherwise
         """
-        if k == 'scanfile' or k == 'data':
+        if k in ['scanfile', 'data', 'h5data']:
             self.params[k] = v
             return True
         return False
