@@ -17,12 +17,15 @@ from numpy import pi
 from scipy.fftpack import fftn, ifftn, fftshift
 from scipy.ndimage import zoom
 from scipy.ndimage.interpolation import shift
-# from scipy.ndimage.fourier import fourier_shift
 from pynx.utils.matplotlib import pyplot as plt
+from matplotlib.figure import Figure
+from matplotlib.backends.backend_agg import FigureCanvas
 from ..utils.plot_utils import showCplx
-from ..utils.pattern import get_img, spiral_fermat, spiral_archimedes
-from .shape import get_view_coord
-from ..wavefront import Wavefront, PropagateNearField, PropagateFarField, FreePU
+from ..utils.pattern import get_img, spiral_fermat, spiral_archimedes, siemens_star
+from .shape import get_view_coord, calc_obj_shape
+from pynx.utils.array import rebin
+from .. import wavefront
+from .. import ptycho
 
 
 def gauss2D(im_size=(64, 64), mu=(0, 0), sigma=(1, 1), rotation=0):
@@ -92,11 +95,6 @@ def make_beam_stop(im_size=(256, 256), radius=10):
     return xx ** 2 + yy ** 2 >= radius ** 2
 
 
-def rebin(a, npix):
-    sh = a.shape[0] / npix, npix, a.shape[1] / npix, npix
-    return a.reshape(sh).sum(-1).sum(1)
-
-
 def tukeywin(window_length, alpha=0.5):
     '''The Tukey window, also known as the tapered cosine window, can be regarded as a cosine lobe of width \alpha * N / 2
     that is convolved with a rectangle window of width (1 - \alpha / 2). At \alpha = 1 it becomes rectangular, and
@@ -130,7 +128,7 @@ def tukeywin(window_length, alpha=0.5):
 
 def tukeywin2D(window_size, alpha=0.5):
     w1 = tukeywin(window_size[0], alpha)
-    w2 = tukeywin(window_size[0], alpha)
+    w2 = tukeywin(window_size[1], alpha)
     return np.outer(w1, w2)
 
 
@@ -241,16 +239,17 @@ class Simulation(object):
 
         probe: specific value of probe can be passed (2D complex numpy array). probe_info can be passed as empty dicitionary (default).
 
-        scan_info: dictionary with  scan_info['type'] = ('spiral' ,'raster') # type of the scan
+        scan_info: dictionary with  scan_info['type'] = ('spiral' ,'raster', 'custom') # type of the scan
                                     scan_info['scan_step_pix'] = value # step in the scan in [nm]
                                     scan_info['n_scans'] = value # number of scan positions
-                                    scan_info['integer_values'] = True # integer values of the scan positions
+                                    scan_info['integer_values'] = True # Require integer positions
+                                    scan_info['x']=value  # pixel coordinates for a custom scan (same for y)
                                     defaults set in update_default_scan
 
         scan: specific value of scan can be passed (list/tuple of two arrays (posx, posy)). scan_info can be passed as empty dictionary (default).
 
         data_info: dictionary with data_info['pix_size_direct_nm'] = value # pixels size in the object space [nm]
-                                    data_info['num_phot_max'] = value #number of photons in maximum pixel
+                                    data_info['nb_photons_per_frame'] = value # average nb photons per frame
                                     data_info['bg'] = value # uniform background added to each frame
                                     data_info['beam_stop_transparency'] = value
                                     data_info['noise'] = 'poisson' will create Poisson noise in the data. None will make noise-free data.
@@ -261,10 +260,10 @@ class Simulation(object):
                                     defaults set in update_default_data
 
         """
-        if prec is 'single':  # precision of the simulation
+        if prec == 'single':  # precision of the simulation
             self.DTYPE_REAL = np.float32  # dtype of the real arrays
             self.DTYPE_CPLX = np.complex64  # dtype of the complex arrays
-        elif prec is 'double':
+        elif prec == 'double':
             self.DTYPE_REAL = np.float64
             self.DTYPE_CPLX = np.complex128
         # print('Using %s precision for simulation.' % prec)
@@ -295,7 +294,8 @@ class Simulation(object):
     def update_default_data(self):
         # print('Updating defaults values for simulation.')
         default_data_info = {'pix_size_direct_nm': 10,
-                             'num_phot_max': 1e9,
+                             'num_phot_max': None,  # Deprecated
+                             'nb_photons_per_frame': 1e8,
                              'bg': 0,
                              'beam_stop_transparency': 0,
                              'noise': 'poisson'}
@@ -377,9 +377,14 @@ class Simulation(object):
         else:
             rf = 1
         s_a = int(s_v[0] // rf), int(s_v[1] // rf)
-        self.make_obj_true(posy_max, posx_max)  # pads the obj with zeros if necessary
+
+        # Make the real object, zoom to fill if necessary
+        nyo, nxo = calc_obj_shape(posx, posy, probe_shape=self.probe.values.shape)
+        self.make_obj_true((nyo, nxo))
+
         self.psi = Im(np.zeros((n, s_v[0], s_v[1]), dtype=self.DTYPE_CPLX))
-        intensity = np.zeros((n, s_a[0], s_a[1]), dtype=self.DTYPE_REAL)
+        intensity = np.ones((n, s_a[0], s_a[1]), dtype=self.DTYPE_REAL)
+
         if 'beam_stop_radius' in self.data_info:
             if 'beam_stop_transparency' not in self.data_info:
                 self.data_info['beam_stop_transparency'] = 0
@@ -387,76 +392,75 @@ class Simulation(object):
             beam_stop_tmp = disc(self.probe.values.shape, self.data_info['beam_stop_radius'])
             beam_stop = self.data_info['beam_stop_transparency'] * (beam_stop_tmp) + (1. - beam_stop_tmp)
         else:
-            beam_stop = 1
-        # TODO: use a single wavefront ? May run out of memory for large simulations
-        for sy, sx, index in zip(posy, posx, list(range(n))):
-            if self.verbose:
-                if (n - index) % 10 == 0 or index == 0:
-                    sys.stdout.write('%d ' % (n - index))
-                    sys.stdout.flush()
-            self.psi.values[index] = psi(self.obj.values, self.probe.values, sx, sy, self.obj.info['border_pix'])
-            self.wd = Wavefront(d=fftshift(self.psi.values[index]), wavelength=wavelength,
-                                pixel_size=pixel_size_object)
-            if near_field:
-                self.wd = PropagateNearField(dz=detector_distance) * self.wd
-            else:
-                self.wd = PropagateFarField(dz=detector_distance) * self.wd
-            intensity_tmp = abs(self.wd.get(shift=True)) ** 2
-            intensity_tmp *= beam_stop
+            beam_stop = np.ones((s_v[0], s_v[1]), dtype=self.DTYPE_REAL)
 
-            if rf > 1:  # rebin_factor
-                intensity[index] = rebin(intensity_tmp, rf)
-                if (self.verbose and index == (len(posx) - 1)):
-                    print("\nBinning data by %s" % rf)
-            else:
-                intensity[index] = intensity_tmp
+        nb_photons_per_frame = self.data_info['nb_photons_per_frame']
+
+        # Use Ptycho code for simulation
+        pd = ptycho.PtychoData(iobs=intensity, positions=(posx * pixel_size_object, posy * pixel_size_object),
+                               detector_distance=detector_distance, wavelength=wavelength, near_field=near_field,
+                               mask=np.isclose(beam_stop, 0), pixel_size_detector=pixel_size_detector)
+        p = ptycho.Ptycho(probe=self.probe.values, obj=self.obj.values, data=pd)
+        p = ptycho.Calc2Obs(nb_photons_per_frame=nb_photons_per_frame,
+                            poisson_noise=(self.data_info['noise'] == 'poisson')) * p
+        intensity = np.fft.fftshift(p.data.iobs, axes=(1, 2)) * beam_stop
+        if rf > 1:  # rebin_factor
+            intensity = rebin(intensity, (1, rf, rf), normalize=False)
+            if self.verbose:
+                print("\nBinning data by %s" % rf)
 
         if self.verbose:
             print("\n")
 
-        intensity /= intensity.max()
-        intensity *= self.data_info['num_phot_max']
+        if self.data_info['num_phot_max'] is not None:
+            intensity /= intensity.max()
+            intensity *= self.data_info['num_phot_max']
         intensity += self.data_info['bg']
-        if self.data_info['noise'] == 'poisson':
-            intensity = np.random.poisson(intensity)
+
         self.amplitude = Im(np.sqrt(intensity))
 
         if self.verbose:
             self.print_info()
 
-    def make_obj_true(self, posy_max, posx_max):
+    def make_obj_true(self, obj_shape):
         """
-        Enlarge object according to scan positions and probe size. Replace previous object.
-        :param posy_max: maximum absolute displacement along x-axis
-        :param posx_max: maximum absolute displacement along y-axis
+        Enlarge object according to real object shape. Replace previous object.
+        :param obj_shape: maximum absolute displacement along x-axis
         :return:
         """
-        info = {'border_pix': 2}  # enlarge on each side (useful for later position refinement)
         o_s = self.obj.values.shape
-        p_s = self.probe.values.shape
-        border = info['border_pix']
-        ot_s = int(2 * (posy_max + p_s[0] // 2 + border)), int(2 * (posx_max + p_s[1] // 2 + border))
-        # print("posx_max=%d, p_s[0] // 2=%d, border=%d" % (posx_max, p_s[0] // 2, border), ot_s)
-        ot_s = max(ot_s[0], o_s[0]), max(ot_s[1], o_s[1])  # the case of very large object ot_s<o_s
-        # print("posy_max=%d, p_s[1] // 2=%d, border=%d" % (posy_max, p_s[1] // 2, border), ot_s)
-
-        if False:
-            # Fill in center with image
-            tmp = np.zeros(ot_s, np.dtype(self.obj.values.dtype))
-            start = ot_s[0] // 2 - o_s[0] // 2, ot_s[1] // 2 - o_s[1] // 2
-            tmp[start[0]:start[0] + o_s[0], start[1]: start[1] + o_s[1]] = self.obj.values
+        if self.obj.info['type'].lower() == 'siemens':
+            mxy = max(obj_shape)
+            tmp = siemens_star(dsize=mxy, nb_rays=36, r_max=0.9 * mxy, nb_rings=8,
+                               cheese_hole_spiral_period=40, cheese_holes_nb=500 * (mxy // 500) ** 2,
+                               cheese_hole_max_radius=6)[:obj_shape[0], :obj_shape[1]]
+            # Make this a transmission object
+            tmp = np.exp((-0.05 + 1j) * tmp).astype(self.obj.values.dtype)
         else:
-            oa, op = np.abs(self.obj.values), np.angle(self.obj.values)
-            # Zoom image
-            tmpa = zoom(oa, (ot_s[0] / o_s[0], ot_s[1] / o_s[1]))
-            tmpp = zoom(op, (ot_s[0] / o_s[0], ot_s[1] / o_s[1]))
-            tmp = (tmpa * np.exp(1j * tmpp)).astype(self.obj.values.dtype)
-        old_info = self.obj.info
-        self.obj = Im(tmp, info)
-        if len(old_info):
-            self.obj.info = old_info
-            if 'border_pix' not in self.obj.info:
-                self.obj.info['border_pix'] = 2
+            if True:
+                # Zoom the object to fill the area
+                oa, op = np.abs(self.obj.values), np.angle(self.obj.values)
+                # Zoom image
+                tmpa = zoom(oa, (obj_shape[0] / o_s[0], obj_shape[1] / o_s[1]), order=0)
+                tmpp = zoom(op, (obj_shape[0] / o_s[0], obj_shape[1] / o_s[1]), order=0)
+                tmp = (tmpa * np.exp(1j * tmpp)).astype(self.obj.values.dtype)
+            else:
+                # Tile the object to fill the area - deactivated as the original image is 512 pixels, so
+                # a size likely similar to the probe, so may produce weird periodic artefacts
+                ny, nx = obj_shape
+                ny0, nx0 = self.obj.values.shape
+                tmp = np.tile(self.obj.values, (int(np.ceil(ny / ny0)), int(np.ceil(nx / nx0))))
+                ny1, nx1 = tmp.shape
+                tmp = tmp[ny1 // 2 - ny // 2:ny1 // 2 - ny // 2 + ny,
+                      nx1 // 2 - nx // 2:nx1 // 2 - nx // 2 + nx]
+
+        if 'alpha_win' in self.obj.info:
+            s = tmp.shape
+            mi = abs(tmp).min()
+            w = tukeywin2D(s, self.obj.info['alpha_win'])
+            tmp = mi + w * (tmp - mi)  # tuckey window to smooth edges (alpha_win)
+
+        self.obj = Im(tmp, self.obj.info)
 
     def make_obj(self):
         self.update_default_obj()
@@ -469,17 +473,19 @@ class Simulation(object):
             obj = self.obj.values
 
         elif ('ampl' in obj_type) and ('phase' in obj_type):
-            im0 = self.DTYPE_REAL(get_img(0))
-            im1 = self.DTYPE_REAL(get_img(1))
+            im0 = np.flipud(self.DTYPE_REAL(get_img(0)))
+            im1 = np.flipud(self.DTYPE_REAL(get_img(1)))
             # Stretch the phase to interval (-phase_stretch/2, +phase_stretch/2)
             phase0 = im1 - im1.min()
             ps = 2 * np.pi
             phase_stretch = ps * phase0 / self.DTYPE_REAL(phase0.max()) - ps / 2.
+            # Let's limit the amplitude extent to 0.8-1
+            im0 = 1.25 ** (-im0.astype(np.float32) / im0.max())
             obj = im0 * np.exp(1j * phase_stretch)
 
         elif ('real' in obj_type) and ('imag' in obj_type):
-            im0 = self.DTYPE_REAL(get_img(0))
-            im1 = self.DTYPE_REAL(get_img(1))
+            im0 = np.flipud(self.DTYPE_REAL(get_img(0)))
+            im1 = np.flipud(self.DTYPE_REAL(get_img(1)))
             obj = im0 + 1j * im1
 
         elif obj_type.lower() == 'random':
@@ -503,6 +509,28 @@ class Simulation(object):
         elif obj_type.lower() == 'flat':
             obj = np.ones(info['shape']).astype(self.DTYPE_CPLX)
 
+        elif obj_type.lower() == 'siemens':
+            obj = siemens_star(dsize=512, nb_rays=36, r_max=450, nb_rings=8, cheese_hole_spiral_period=200)
+        elif obj_type.lower() == 'logo':
+            fig = Figure(figsize=(6, 4), dpi=128)
+            canvas = FigureCanvas(fig)
+            ax = fig.subplots()
+            t = ax.text(0.5, 0.5, "PyNX", fontsize=140, fontweight='heavy', ha='center', va='center')
+            ax.axis('off')
+            canvas.draw()
+            a = np.array(canvas.renderer.buffer_rgba())[:, :, 0]
+
+            fig = Figure(figsize=(6, 4), dpi=128)
+            canvas = FigureCanvas(fig)
+            ax = fig.subplots()
+            t = ax.text(0.5, 0.5, "PyNX", fontsize=150, fontweight='heavy', ha='center', va='center')
+            ax.axis('off')
+            canvas.draw()
+            ph = np.array(canvas.renderer.buffer_rgba())[:, :, 0]
+            ph = (ph.max() - ph) / ph.max()
+
+            obj = np.flipud((a.max() - a) * np.exp(-1j * ph).astype(self.DTYPE_CPLX))
+
         else:
             msg = "Unknown object type:", self.obj.info['type']
             raise NameError(msg)
@@ -512,7 +540,7 @@ class Simulation(object):
             phase0 = phase - phase.min()
             if phase0.any():
                 ps = info['phase_stretch']
-                phase_stretch = ps * phase0 / self.DTYPE_REAL(phase0.max()) - ps / 2.
+                phase_stretch = ps * phase0 / phase0.max() - ps / 2.
                 obj = abs(obj) * np.exp(1j * phase_stretch)
 
         if 'ampl_range' in info and not 'range' in info:
@@ -570,7 +598,7 @@ class Simulation(object):
             # Pixel size at aperture
             pixel_size_aperture = self.wavelength * focal_length / (nx * pixel_size_focus)
             d = np.zeros((nx, nx), dtype=np.complex64)
-            self.w = Wavefront(d=d, wavelength=self.wavelength, pixel_size=pixel_size_aperture)
+            self.w = wavefront.Wavefront(d=d, wavelength=self.wavelength, pixel_size=pixel_size_aperture)
             x, y = self.w.get_x_y()
             if len(info['aperture']) == 1:
                 # Circular aperture
@@ -581,11 +609,11 @@ class Simulation(object):
                 # Rectangular slit
                 widthx, widthy = info['aperture']
                 self.w.set((abs(y) < (widthy / 2)) * (abs(x) < (widthx / 2)))
-            w = PropagateFarField(focal_length, forward=False) * self.w
+            w = wavefront.PropagateFarField(focal_length, forward=False) * self.w
             if 'defocus' in info.keys():
-                w = PropagateNearField(info['defocus']) * self.w
+                w = wavefront.PropagateNearField(info['defocus']) * self.w
             self.probe.values = self.w.get(shift=True)
-            w = FreePU() * w
+            w = wavefront.FreePU() * w
 
         elif probe_type == 'near_field':
             defocus = self.probe.info['defocus']
@@ -598,7 +626,7 @@ class Simulation(object):
 
             # Pixel size at aperture
             d = np.zeros((nx, nx), dtype=np.complex64)
-            self.w = Wavefront(d=d, wavelength=self.wavelength, pixel_size=pixel_size_detector)
+            self.w = wavefront.Wavefront(d=d, wavelength=self.wavelength, pixel_size=pixel_size_detector)
             x, y = self.w.get_x_y()
             if len(info['aperture']) == 1:
                 # Circular aperture
@@ -609,9 +637,9 @@ class Simulation(object):
                 # Rectangular slit
                 widthx, widthy = info['aperture']
                 self.w.set((abs(y) < (widthy / 2)) * (abs(x) < (widthx / 2)))
-            w = PropagateNearField(dz=defocus) * self.w
+            w = wavefront.PropagateNearField(dz=defocus) * self.w
             self.probe.values = self.w.get(shift=True)
-            w = FreePU() * w
+            w = wavefront.FreePU() * w
 
         else:
             msg = "Unknown probe type:", self.probe.info['type']
@@ -639,6 +667,9 @@ class Simulation(object):
             n = info['n_scans']
             dmax = 2 * np.sqrt(n / np.pi) * info['scan_step_pix']
             posy, posx = spiral_fermat(dmax, n)
+        elif scan_type == 'custom':
+            posx = info['x']
+            posy = info['y']
         else:
             msg = "Unknown scan type:", self.scan.info['type']
             raise NameError(msg)
