@@ -73,7 +73,7 @@ def make_ptycho_data(dsize=256, nb_frame=100, nb_photons=1e9):
 
 def make_ptycho_data_cxi(dsize=256, nb_frame=100, nb_photons=1e9, dir_name=None):
     iobs, pixel_size_detector, wavelength, detector_distance, x, y = make_ptycho_data(dsize, nb_frame, nb_photons)
-    f, path = tempfile.mkstemp(suffix='.cxi', dir=dir_name)
+    f, path = tempfile.mkstemp(suffix='.cxi', prefix="TestPtycho", dir=dir_name)
     save_ptycho_data_cxi(path, iobs, pixel_size_detector, wavelength, detector_distance, x, y, z=None,
                          monitor=None, mask=None, instrument="Simulation", overwrite=True)
     return path
@@ -154,7 +154,7 @@ class TestPtycho(unittest.TestCase):
         self.make_ptycho_obj()
         o = self.p.get_obj()
         pr = self.p.get_probe()
-        f, path = tempfile.mkstemp(suffix='.cxi', dir=self.tmp_dir)
+        f, path = tempfile.mkstemp(suffix='.cxi', prefix="TestPtycho", dir=self.tmp_dir)
         self.p.save_obj_probe_cxi(path)
         self.p.load_obj_probe_cxi(path, verbose=False)
         self.o = o
