@@ -167,7 +167,7 @@ __global__ void CUDA_fhkl(float *fhkl_real,float *fhkl_imag,
                         const float *vh,const float *vk,const float *vl)
 {
    #define BLOCKSIZE %d
-   #define twopi 6.2831853071795862f
+   #define twopi -6.2831853071795862f
    const unsigned long ix=threadIdx.x+blockDim.x*blockIdx.x;
    const float h=twopi*vh[ix];
    const float k=twopi*vk[ix];
@@ -222,7 +222,7 @@ __global__ void CUDA_fhklo(float *fhkl_real,float *fhkl_imag,
                         const float *vh,const float *vk,const float *vl)
 {
    #define BLOCKSIZE %d
-   #define twopi 6.2831853071795862f
+   #define twopi -6.2831853071795862f
    const unsigned long ix=threadIdx.x+blockDim.x*blockIdx.x;
    const float h=twopi*vh[ix];
    const float k=twopi*vk[ix];
@@ -278,7 +278,7 @@ __global__ void CUDA_fhklo_grazing(float *fhkl_real,float *fhkl_imag,
                         const float *vkx,const float *vky,const float *vkzr,const float *vkzi)
 {
    #define BLOCKSIZE %d
-   #define twopi 6.2831853071795862f
+   #define twopi -6.2831853071795862f
    const unsigned long ix=threadIdx.x+blockDim.x*blockIdx.x;
    const float kx=twopi*vkx[ix];
    const float ky=twopi*vky[ix];
@@ -338,7 +338,7 @@ __global__ void CUDA_fhklo_grazing(float *fhkl_real,float *fhkl_imag,
 
 
 code_CPU_fhkl_xyz = """
-  const float PI2         = 6.28318530717958647692528676655900577f;
+  const float PI2         = -6.28318530717958647692528676655900577f;
   for(unsigned long i=0;i<nhkl;i++)
   {
       float fr=0,fi=0;
@@ -375,7 +375,7 @@ code_CPU_fhkl_xyz = """
 """
 
 code_CPU_fhklo_xyz = """
-  const float PI2         = 6.28318530717958647692528676655900577f;
+  const float PI2         = -6.28318530717958647692528676655900577f;
   for(unsigned long i=0;i<nhkl;i++)
   {
       float fr=0,fi=0;
@@ -414,7 +414,7 @@ code_CPU_fhklo_xyz = """
 """
 
 code_CPU_fhkl_grazing_xyz = """
-  const float PI2         = 6.28318530717958647692528676655900577f;
+  const float PI2         = -6.28318530717958647692528676655900577f;
   for(unsigned long i=0;i<nhkl;i++)
   {
       float fr=0,fi=0;
@@ -453,7 +453,7 @@ code_CPU_fhkl_grazing_xyz = """
 """
 
 code_CPU_fhklo_grazing_xyz = """
-  const float PI2         = 6.28318530717958647692528676655900577f;
+  const float PI2         = -6.28318530717958647692528676655900577f;
   for(unsigned long i=0;i<nhkl;i++)
   {
       float fr=0,fi=0;
@@ -505,7 +505,7 @@ void Fhkl(__global float *fhkl_real,__global float *fhkl_imag,
                    __global float *vh,__global float *vk,__global float *vl)
 {
    #define BLOCKSIZE %(block_size)d
-   #define twopi 6.2831853071795862f
+   #define twopi -6.2831853071795862f
    // Block index
    int bx = get_group_id(0);
    int by = get_group_id(1);
@@ -569,7 +569,7 @@ __kernel void Fhkl(__global float *fhkl_real,__global float *fhkl_imag,
                    __global float *vh,__global float *vk,__global float *vl)
 {
    #define BLOCKSIZE %(block_size)d
-   #define twopi 6.2831853071795862f
+   #define twopi -6.2831853071795862f
    // Block index
    int bx = get_group_id(0);
    int by = get_group_id(1);
@@ -637,7 +637,7 @@ void Fhkl(__global float *fhkl_real,__global float *fhkl_imag,
                    __global float *vkx,__global float *vky,__global float *vkzr,__global float *vkzi)
 {
    #define BLOCKSIZE %(block_size)d
-   #define twopi 6.2831853071795862f
+   #define twopi -6.2831853071795862f
    // Block index
    int bx = get_group_id(0);
    int by = get_group_id(1);
@@ -704,7 +704,7 @@ __kernel void Fhkl(__global float *fhkl_real,__global float *fhkl_imag,
                    __global float *vkx,__global float *vky,__global float *vkzr,__global float *vkzi)
 {
    #define BLOCKSIZE %(block_size)d
-   #define twopi 6.2831853071795862f
+   #define twopi -6.2831853071795862f
    // Block index
    int bx = get_group_id(0);
    int by = get_group_id(1);
@@ -1257,8 +1257,8 @@ class OpenCLThread_Fhkl(threading.Thread):
 def Fhkl_thread(h, k, l, x, y, z, occ=None, verbose=False, gpu_name="GTX 295", nbCPUthread=None, sz_imag=None,
                 language="OpenCL", cl_platform=""):
     """
-    Compute          F(hkl)=SUM_i exp(2j*pi*(h*x_i  + k*y_i  + l*z_i))
-    or equivalently: F(k)  =SUM_i exp(2j*pi*(sx*x_i + sy*y_i + sz*z_i))
+    Compute          F(hkl)=SUM_i exp(-2j*pi*(h*x_i  + k*y_i  + l*z_i))
+    or equivalently: F(k)  =SUM_i exp(-2j*pi*(sx*x_i + sy*y_i + sz*z_i))
 
     nbCPUthread can be used only for CPU computing, when no GPU is available. nbCPUthread can
     be set to the number of cores available. Using None (the default) makes the program recognize
@@ -1421,5 +1421,5 @@ def Fhkl_gold(h, k, l, x, y, z, occ=None, verbose=False, dtype=numpy.float64):
     if type(occ) == type(None):
         occ = numpy.float32(1.0)
     for i in range(len(vh.flat)):
-        fhkl.flat[i] = (occ * numpy.exp(2j * numpy.pi * (vh.flat[i] * vx + vk.flat[i] * vy + vl.flat[i] * vz))).sum()
+        fhkl.flat[i] = (occ * numpy.exp(-2j * numpy.pi * (vh.flat[i] * vx + vk.flat[i] * vy + vl.flat[i] * vz))).sum()
     return fhkl, time.time() - t0
