@@ -9,6 +9,7 @@
 import sys
 import io
 import unittest
+import warnings
 from pynx.test import *
 
 help_text = """
@@ -105,4 +106,6 @@ def run_tests():
 
 
 if __name__ == '__main__':
-    run_tests()
\ No newline at end of file
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore')
+        run_tests()
\ No newline at end of file
