@@ -13,6 +13,7 @@ This package provides access to the full PyNX test suite.
 import unittest
 import sys
 import io
+from pynx.test.test_imports import suite as imports_suite
 from pynx.processing_unit.test import suite as processing_unit_suite
 from pynx.cdi.test import suite as cdi_suite
 from pynx.cdi.runner.test import suite as cdi_runner_suite
@@ -22,6 +23,7 @@ from pynx.ptycho.runner.test import suite as ptycho_runner_suite
 
 def suite():
     test_suite = unittest.TestSuite()
+    test_suite.addTest(imports_suite())
     test_suite.addTest(processing_unit_suite())
     test_suite.addTest(cdi_suite())
     test_suite.addTest(cdi_runner_suite())
