@@ -10,11 +10,14 @@ import socket
 import time
 import timeit
 import sys
+import os
+import subprocess
 import gc
 import sqlite3
 import numpy as np
 from pynx.utils.matplotlib import pyplot as plt
 from pynx.processing_unit import has_cuda, has_opencl, default_processing_unit
+from pynx.utils.math import test_smaller_primes
 
 if has_opencl:
     from pynx.processing_unit import opencl_device, cl_processing_unit
@@ -27,6 +30,7 @@ if has_cuda:
     import pycuda.driver as cu_drv
     import pycuda.gpuarray as cua
     from pynx.processing_unit.cuda_device import cuda_device_fft_speed
+    import skcuda.fft as cu_fft
 from pynx.scattering.test import mrats
 
 
@@ -35,7 +39,7 @@ class SpeedTest(object):
     Class for speed tests using either GPU or CPU
     """
 
-    def __init__(self, gpu_name, language, cl_platform=None):
+    def __init__(self, gpu_name, language, cl_platform=None, verbose=True):
         """
 
         :param gpu_name: the gpu name to be tested, or 'CPU'.
@@ -94,7 +98,9 @@ class SpeedTest(object):
 
         for k, v in self.results.items():
             try:
-                self.db_conn.execute('ALTER TABLE pynx_speed_test ADD COLUMN %s;' % (k))
+                k = k.replace("*", "_")
+                k = k.replace("=", "_")
+                self.db_conn.execute('ALTER TABLE pynx_speed_test ADD COLUMN %s;' % k)
             except:
                 # column already existed
                 pass
@@ -189,13 +195,10 @@ class SpeedTest(object):
                     queue_h2d.finish()
                     queue_d2h.finish()
                     dt1 = timeit.default_timer() - t0
-                    print("Test #%2d: dt=%7.3fs" % (i, dt1))
                     if dt == 0 or dt1 < dt:
                         dt = dt1
 
-                gbytes = size * 4 * nb * multiplicity / dt / 1024 ** 3
-                if parallel:
-                    gbytes *= 2
+                gbytes = 2 * size * 4 * nb * multiplicity / dt / 1024 ** 3
 
                 for a in vad:
                     a.data.release()
@@ -270,12 +273,9 @@ class SpeedTest(object):
                     cu_stream_in.synchronize()
                     cu_stream_out.synchronize()
                     dt1 = timeit.default_timer() - t0
-                    print("Test #%2d: dt=%7.3fs" % (i, dt1))
                     if dt == 0 or dt1 < dt:
                         dt = dt1
-                gbytes = size * 4 * nb * multiplicity / dt / 1024 ** 3
-                if parallel:
-                    gbytes *= 2
+                gbytes = 2 * size * 4 * nb * multiplicity / dt / 1024 ** 3
 
                 if verbose:
                     print("Free memory...")
@@ -471,6 +471,82 @@ class SpeedTest(object):
         print('%30s: %8.2f Gbyte/s, dt =%6.4fs' % (s, gbytes, dt))
         return dt
 
+    def test_fft_host2device_swap(self, size, n_fft=1, n_stack=3, n_iter=3):
+        """
+        Test FFT calculations while constantly swapping datasets in pinned memory.
+        :param size: the size N of the 2D FFT which will be computed (using 16xNxN arrays)
+        :param n_fft: the number of FFT + iFFT to perform between swaps. Depending on the relative speed
+                       of memory transfer and FFT, a minimum number of FFT must be done before saturating
+                       the GPU.
+        :param n_iter: number of iterations for the calculations
+        :param n_stack: number of stacks of 16xNxN to be swapped (must be>=3)
+        :return: the time to perform the calculations with swap
+        """
+        if 'cuda' not in self.language:
+            return 0
+        cu_stream_fft = cu_drv.Stream()
+        cu_stream_in = cu_drv.Stream()
+        cu_stream_out = cu_drv.Stream()
+
+        # Create data with pinned memory, random data to avoid any smart optimisation
+        vpsi = []
+        for j in range(n_stack):
+            vpsi.append(cu_drv.pagelocked_empty((16, size, size), np.complex64))
+            vpsi[-1][:] = np.random.uniform(0, 1, (16, size, size))
+        # Allocate 3 arrays in GPU
+        cu_psi = cua.to_gpu(vpsi[0])
+        cu_psi_in = cua.to_gpu(vpsi[1])
+        cu_psi_out = cua.to_gpu(vpsi[2])
+
+        # First test fft on array remaining in GPU
+        plan = self.pu.cu_fft_get_plan(cu_psi, batch=True, stream=cu_stream_fft)
+        t0 = timeit.default_timer()
+        for i in range(n_iter * n_stack):
+            for k in range(n_fft):
+                cu_fft.fft(cu_psi, cu_psi, plan, scale=False)
+                cu_fft.ifft(cu_psi, cu_psi, plan, scale=False)
+
+        self.pu.finish()
+        dt0 = timeit.default_timer() - t0
+        # This measures the number of Gbyte/s for which the n_fft FFT are calculated
+        # If n_fft=0 this should correspond to the total bandwidth
+        gbytes0 = cu_psi.size * 8 * 2 * n_fft * n_iter * n_stack / dt0 / 1024 ** 3
+
+        # Now perform FFT while transferring in // data to and from GPU with three queues
+        t0 = timeit.default_timer()
+        for i in range(n_iter):
+            for j in range(n_stack):
+                cu_drv.memcpy_htod_async(dest=cu_psi_in.gpudata, src=vpsi[(j + 1) % n_stack],
+                                         stream=cu_stream_in)
+                for k in range(n_fft):
+                    cu_fft.fft(cu_psi, cu_psi, plan, scale=False)
+                    cu_fft.ifft(cu_psi, cu_psi, plan, scale=False)
+                cu_drv.memcpy_dtoh_async(src=cu_psi_out.gpudata, dest=vpsi[(j - 1) % n_stack],
+                                         stream=cu_stream_out)
+                # Swap stacks
+                cu_psi_in, cu_psi, cu_psi_out = cu_psi_out, cu_psi_in, cu_psi
+                # Make sure tasks are finished in each stream before beginning a new one.
+                # Use events so that the wait is done asynchronously on the GPU
+                ev_fft = cu_drv.Event(cu_drv.event_flags.DISABLE_TIMING)
+                ev_in = cu_drv.Event(cu_drv.event_flags.DISABLE_TIMING)
+                ev_out = cu_drv.Event(cu_drv.event_flags.DISABLE_TIMING)
+                ev_fft.record(cu_stream_fft)
+                ev_in.record(cu_stream_in)
+                ev_out.record(cu_stream_out)
+                cu_stream_fft.wait_for_event(ev_in)  # Data must be arrived before being processed
+                cu_stream_in.wait_for_event(ev_out)  # Data out must be finished before replacing by in
+                cu_stream_out.wait_for_event(ev_fft)  # Processing must be finished before transfer out
+        self.pu.finish()
+        dt1 = timeit.default_timer() - t0
+        gbytes1 = cu_psi.size * 8 * 2 * n_iter * n_fft * n_stack / dt1 / 1024 ** 3
+        print("Time for %3d FFT of size 16x%dx%d:\n"
+              "             on-GPU:%6.3fs (%8.2f Gbyte/s)\n"
+              "      with h2d swap:%6.3fs (%8.2f Gbyte/s) (%2d FT+iFT per swap)" %
+              (n_iter * n_stack * n_fft, size, size, dt0, gbytes0, dt1, gbytes1, n_fft))
+        self.results['fft_h2d_swap_%dfft%d_Gbytes' % (n_fft, size)] = gbytes1
+        self.results['fft_h2d_swap_%dfft%d_dt' % (n_fft, size)] = dt1
+        return dt1
+
     def test_scattering(self, size):
         """
         Test using pynx.scattering.speed.mrats
@@ -487,54 +563,53 @@ class SpeedTest(object):
         self.results['scattering_%d_dt' % (size)] = dt
         return dt
 
-    def test_fft_2d(self, size):
+    def test_fft_2d(self, size, stack_size=32, verbose=True, nb_cycle=1):
         """
         Test a stacked (16) 2D FFT
         :param size: the size N of the 2D FFT, which will be computed as using 16xNxN array
-        :return: nothing. The result is added to self.results
+        :return: dt=the time for one FT+FT-1. The result is added to self.results
         """
         gflops = -1
         dt = 0
-        try:
-            if 'opencl' in self.language:
-                gflops, dt = cl_device_fft_speed(self.pu.cl_device, fft_shape=(16, size, size), axes=(-1, -2),
-                                                 verbose=False, timing=True)
-            elif 'cuda' in self.language:
-                gflops, dt = cuda_device_fft_speed(self.pu.cu_device, fft_shape=(16, size, size), batch=True,
-                                                   verbose=False, timing=True)
-            else:
-                # TODO: test CPU speed
-                pass
-        except:
+        if 'opencl' in self.language:
+            gflops, dt, ax = cl_device_fft_speed(self.pu.cl_device, fft_shape=(stack_size, size, size), axes=(-1, -2),
+                                                 verbose=False, timing=True, shuffle_axes=True, nb_cycle=nb_cycle)
+        elif 'cuda' in self.language:
+            gflops, dt = cuda_device_fft_speed(self.pu.cu_device, fft_shape=(stack_size, size, size), batch=True,
+                                               verbose=False, timing=True, nb_cycle=nb_cycle)
+        else:
+            # TODO: test CPU speed
             pass
-        self.results['fft_2Dx16_%d_Gflops' % (size)] = gflops
-        self.results['fft_2Dx16_%d_dt' % (size)] = dt
-        print('fft_2Dx16_%d: %8.2f Gflop/s, dt =%6.4fs' % (size, gflops, dt))
+        self.results['fft_2Dx%d_%d_Gflops' % (stack_size, size)] = gflops
+        self.results['fft_2Dx%d_%d_dt' % (stack_size, size)] = dt
+        if verbose:
+            print('fft_2Dx%d_%d: %8.2f Gflop/s, dt =%6.4fs' % (stack_size, size, gflops, dt))
         return dt
 
-    def test_fft_3d(self, size):
+    def test_fft_3d(self, size, verbose=True, nb_cycle=1):
         """
         Test a 3D FFT
         :param size: the size N of the 3D FFT, which will be computed using a NxNxN array
-        :return: nothing. The result is added to self.results
+        :return: dt=the time for one FT+FT-1. The result is added to self.results
         """
         gflops = -1
         dt = 0
         try:
             if 'opencl' in self.language:
-                gflops, dt = cl_device_fft_speed(self.pu.cl_device, fft_shape=(size, size, size), axes=(-1, -2, -3),
-                                                 verbose=False, timing=True)
+                gflops, dt, ax = cl_device_fft_speed(self.pu.cl_device, fft_shape=(size, size, size), axes=(-1, -2, -3),
+                                                     verbose=False, timing=True, shuffle_axes=True, nb_cycle=nb_cycle)
             elif 'cuda' in self.language:
                 gflops, dt = cuda_device_fft_speed(self.pu.cu_device, fft_shape=(size, size, size), batch=False,
-                                                   verbose=False, timing=True)
+                                                   verbose=False, timing=True, nb_cycle=nb_cycle)
             else:
                 # TODO: test CPU speed
                 pass
         except:
             pass
-        self.results['fft_3D_%d_Gflops' % (size)] = gflops
-        self.results['fft_3D_%d_dt' % (size)] = dt
-        print('fft_3D_%d: %8.2f Gflop/s, dt =%6.4fs' % (size, gflops, dt))
+        self.results['fft_3D_%d_Gflops' % size] = gflops
+        self.results['fft_3D_%d_dt' % size] = dt
+        if verbose:
+            print('fft_3D_%d: %8.2f Gflop/s, dt =%6.4fs' % (size, gflops, dt))
         return dt
 
     def test_ptycho(self, nb_frame, frame_size, nb_cycle=20, algo="AP"):
@@ -609,6 +684,15 @@ class SpeedTest(object):
         p = op ** nb_cycle * p
         ops.default_processing_unit.finish()
         dt = timeit.default_timer() - t0
+
+        # Free memory
+        p = ops.FreePU() * p
+        if 'opencl' in self.language:
+            ops.default_processing_unit.cl_fft_free_plans()
+        elif 'cuda' in self.language:
+            ops.default_processing_unit.cu_fft_free_plans()
+        gc.collect()
+
         # 5 * n * n is a (very) conservative estimate of the number of elementwise operations.
         gflops = nb_cycle * nb_frame * (2 * 5 * n * n * np.log2(n * n) + n * n * 5) / 1e9 / dt
 
@@ -618,12 +702,41 @@ class SpeedTest(object):
 
         return dt
 
-    def test_cdi3d(self, size=128, nb_cycle=20, algo="ER"):
+    def test_ptycho_runner_simul(self, nb_frame, frame_size, algo="ML**100,DM**200,probe=1,nbprobe=1"):
+        """
+        Run 2D ptychography speed test using pynx-simulationpty.py runner
+
+        :param nb_frame: number of frames
+        :param frame_size: frame size in pixels
+        :param algo: algorithm chains to use
+        :param mpi_size: number of parallel MPI process to use [default=1, no MPI]
+        :return: the algorithm execution time (not counting initialisation)
+        """
+
+        my_env = os.environ.copy()
+        my_env["PYNX_PU"] = "%s.%s" % (self.language, self.gpu_name)
+        args = ['pynx-simulationpty.py', 'frame_nb=%d' % nb_frame, 'frame_size=%d' % frame_size,
+                'algorithm=%s' % algo, 'verbose=50', 'saveprefix=none']
+        p = subprocess.Popen(args, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=my_env, shell=False)
+
+        stdout, stderr = p.communicate(timeout=200)
+        res = p.returncode
+        llout = str(stdout).split('\\n')
+        dt = 0
+        for l in llout:
+            if 'Total elapsed time for algorithms' in l:
+                dt += float(l.split()[-1][:-1])
+
+        n = frame_size
+        s = 'ptycho_runner_simul_%dx%dx%d_%s_dt' % (nb_frame, n, n, algo)
+        self.results[s] = dt
+        print('%s: dt =%6.4fs' % (s[:-3], dt))
+
+    def test_cdi3d(self, size=128, algo="ER**20"):
         """
         Run 3D CDI speed test
         :param size: the size of the object along one dimension
-        :param nb_cycle: the number of cycles to execute
-        :param algo: the algorithm to use (ER by default, can also be HIO or RAAR)
+        :param algo: the algorithm string to use (ER**20 by default, can also be HIO or RAAR)
         :return: the execution time (not counting initialisation)
         """
         from pynx.cdi.cdi import CDI
@@ -650,32 +763,41 @@ class SpeedTest(object):
         # Start from a slightly loose support
         support = (abs(x) < 20) * (abs(y) < 20) * (abs(z) < 25)
 
-        cdi = CDI(np.zeros_like(obj0), obj=obj0, support=fftshift(support), mask=None, wavelength=1e-10,
+        cdi = CDI(np.zeros_like(obj0), obj=obj0, support=np.fft.fftshift(support), mask=None, wavelength=1e-10,
                   pixel_size_detector=55e-6)
 
         cdi = ops.Calc2Obs() * cdi
 
-        if algo.lower() == "hio":
-            op = ops.HIO()
-        elif algo.lower() == "raar":
-            op = ops.RAAR()
-        else:
-            algo = "ER"
-            op = ops.ER(calc_llk=False)
-
-        cdi = op ** 5 * cdi
+        # Init object & algorithms
+        cdi = ops.ER() ** 5 * cdi
         ops.default_processing_unit.finish()
 
+        # Prepare real algo
+        er = ops.ER()
+        raar = ops.RAAR()
+        hio = ops.HIO()
+
         t0 = timeit.default_timer()
-        cdi = op ** nb_cycle * cdi
+        cdi = eval(algo.lower()) * cdi
         ops.default_processing_unit.finish()
         dt = timeit.default_timer() - t0
+
+        # Free memory
+        cdi = ops.FreePU() * cdi
+        if 'opencl' in self.language:
+            ops.default_processing_unit.cl_fft_free_plans()
+        elif 'cuda' in self.language:
+            ops.default_processing_unit.cu_fft_free_plans()
+        gc.collect()
+
+        nb_cycle = cdi.cycle - 5
+
         # 5 * n * n is a (very) conservative estimate of the number of elementwise operations.
         gflops = nb_cycle * (2 * 5 * n ** 3 * np.log2(n ** 3) + n * n * n * 5) / 1e9 / dt
 
         self.results['cdi3d_%dx%dx%d_%s_Gflops' % (n, n, n, algo)] = gflops
         self.results['cdi3d_%dx%dx%d_%s_dt' % (n, n, n, algo)] = dt
-        print('cdi3d_%dx%dx%d_%d%s: %8.2f Gflop/s, dt =%6.4fs/cycle' % (n, n, n, nb_cycle, algo, gflops, dt / nb_cycle))
+        print('cdi3d_%dx%dx%d_%s: %8.2f Gflop/s, dt =%6.4fs/cycle' % (n, n, n, algo, gflops, dt / nb_cycle))
 
         return dt
 
@@ -717,7 +839,7 @@ class SpeedTest(object):
         dt = self.test_fft_2d(size=1024)
         if dt < 1:
             dt = self.test_fft_2d(size=2048)
-        if dt < 1 and 'cuda' in self.language:
+        if dt < 1:
             dt = self.test_fft_2d(size=4096)
 
     def run_fft3D(self, verbose=True):
@@ -731,6 +853,39 @@ class SpeedTest(object):
         if dt < 1:
             dt = self.test_fft_3d(size=512)
 
+    def benchmark(self):
+        """
+        Run a series of test for mem, FFT, CDI, ptycho and return a 1-line result
+        :return:
+        """
+        self.test_mem_host2device(2 ** 24, pinned=True, parallel=True, )
+        self.test_mem_copy(2 ** 24)
+        self.test_fft_3d(size=256)
+        self.test_fft_3d(size=512)
+        self.test_fft_3d(size=1024)
+        self.test_fft_2d(size=256, stack_size=32)
+        self.test_fft_2d(size=512, stack_size=32)
+        self.test_fft_2d(size=1024, stack_size=32)
+        self.test_cdi3d(size=512, algo="ER**100*HIO**800")
+        self.test_ptycho_runner_simul(400, 256, algo="ML**100,DM**200,probe=1")
+        self.test_ptycho_runner_simul(400, 512, algo="ML**100,DM**200,probe=1,nbprobe=3")
+
+        res = ("%s" + "\t%6.2f" * 11) % \
+              (self.language,
+               self.results['mem_h2d_parallel_pinned_64M_x1_Gbytes'],
+               self.results['mem_copy_64M_Gbytes'],
+               self.results['fft_3D_256_dt'] * 1000,
+               self.results['fft_3D_512_dt'] * 1000,
+               self.results['fft_3D_1024_dt'] * 1000,
+               self.results['fft_2Dx32_256_dt'] * 1000,
+               self.results['fft_2Dx32_512_dt'] * 1000,
+               self.results['fft_2Dx32_1024_dt'] * 1000,
+               self.results['cdi3d_512x512x512_ER**100*HIO**800_dt'],
+               self.results['ptycho_runner_simul_400x256x256_ML**100,DM**200,probe=1_dt'],
+               self.results['ptycho_runner_simul_400x512x512_ML**100,DM**200,probe=1,nbprobe=3_dt'])
+        print(res)
+        return res
+
     def run(self, tests='all', export_db=None, verbose=True):
         """
         Run selected tests
@@ -752,7 +907,12 @@ class SpeedTest(object):
             if 'ptycho' in s.lower() or s.lower() == 'all':
                 self.test_ptycho(nb_frame=64, frame_size=256, nb_cycle=20, algo='AP')
             if 'cdi' in s.lower() or s.lower() == 'all':
-                self.test_cdi3d(size=128, nb_cycle=20, algo='ER')
+                self.test_cdi3d(size=128, algo='ER**20')
+            if 'swap' in s.lower() or s.lower() == 'all':
+                for n_fft in range(1, 5):
+                    self.test_fft_host2device_swap(size=1024, n_fft=n_fft, n_iter=2, n_stack=5)
+            if 'benchmark' in s.lower():
+                self.benchmark()
         if export_db:
             self.export_db()
 
@@ -770,6 +930,8 @@ class SpeedTest(object):
             else:
                 vals += str(v)
         com = "INSERT INTO pynx_speed_test (%s) VALUES (%s);" % (cols, vals)
+        com = com.replace("*", "_")
+        com = com.replace("=", "_")
         self.db_curs.execute(com)
         self.db_conn.commit()
         self.db_curs.close()
@@ -855,6 +1017,61 @@ class SpeedTest(object):
                 plt.show()
 
 
+def plot_fft_speed(gpu_name, nmin, nmax, ndim=2, stack_size=64, nb_cycle=4):
+    """
+    Plot the FFT vs the size of the ZD FFT
+    :param gpu_name:
+    :param language:
+    :param nmin:
+    :param nmax:
+    :return:
+    """
+    import matplotlib.pyplot as plt
+    langs = []
+    if has_cuda:
+        langs += ['CUDA']
+    if has_opencl:
+        langs += ['OpenCL']
+    res = {}
+    for lang in langs:
+        if 'cl' in lang.lower():
+            print("Testing OpenCL: testing axes permutations to get optimal speed (3-10x slower)")
+        vn = []
+        vdt = []
+        vgbs = []
+        s = SpeedTest(gpu_name=gpu_name, language=lang)
+        max_prime = 7
+        # if 'cl' in lang.lower():
+        #     max_prime = 13
+        for i in range(nmin, nmax):
+            if test_smaller_primes(i, max_prime, required_dividers=None):
+                if ndim == 2:
+                    dt = s.test_fft_2d(size=i, stack_size=stack_size, verbose=False, nb_cycle=nb_cycle)
+                    # 2 pairs of 1D FFT = 8 i/o (4 reads and 4 writes)
+                    gbs = stack_size * i ** 2 * 8 * 8 / 1000 ** 3 / dt
+                    print("%s %dD FFT 64x %3d**%d: dt=%8.5fs, %8.3fGB/s" % (lang, ndim, i, ndim, dt, gbs))
+                else:
+                    dt = s.test_fft_3d(size=i, verbose=False, nb_cycle=nb_cycle)
+                    # 3 pairs of 1D FFT = 6 reads and 6 writes
+                    gbs = i ** 3 * 8 * 12 / 1000 ** 3 / dt
+                    print("%s %dD FFT %3d**%d: dt=%8.5fs, %8.3fGB/s" % (lang, ndim, i, ndim, dt, gbs))
+                vn.append(i)
+                vdt.append(dt)
+                vgbs.append(gbs)
+        res[lang] = np.array(vn, dtype=np.int32), np.array(vdt), np.array(vgbs)
+    plt.figure(figsize=(12, 6))
+    for lang in langs:
+        vn, vdt, vgbs = res[lang]
+        plt.plot(vn, vgbs, 'g.' if 'cl' in lang.lower() else 'b.', label=lang)
+        idx = np.where(abs(np.log2(vn) % 1) < 1e-4)[0]
+        plt.plot(vn[idx], vgbs[idx], 'g.' if 'cl' in lang.lower() else 'b.', markersize=12)
+    plt.ylim(0)
+    plt.ylabel("GB/s")
+    plt.xlabel("FFT size")
+    plt.legend()
+    plt.title("%dD FFT speed" % ndim)
+
+
 if __name__ == '__main__':
     gpu_name = None
     language = None
@@ -866,7 +1083,7 @@ if __name__ == '__main__':
         if len(karg) == 2:
             if karg[0] == 'gpu':
                 gpu_name = karg[1]
-            elif karg[0] == 'language':
+            elif 'lang' in karg[0]:
                 language = karg[1]
             elif karg[0] == 'cl_platform':
                 cl_platform = karg[1]
